#!/usr/bin/python

# Code taken from/based on https://github.com/Piratenfraktion-Berlin/PublicDocs
# thanks to Philip Brechler, @Plaetzchen

import os
from django.conf import settings
from django.utils.encoding import smart_unicode 

from pdfminer.pdfparser import PDFParser, PDFDocument
from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
from pdfminer.converter import PDFPageAggregator
from pdfminer.layout import LAParams, LTTextBox, LTTextLine, LTFigure


class DocScanner():

    def __init__(self, document):
        self.document = document
        self.pdf_file = None
        try:
            self.pdf_path = os.path.join(settings.MEDIA_ROOT, unicode(self.document.document))
            self.pdf_file = open(self.pdf_path, 'rb')
        except IOError:
            # the file doesn't exist or similar problem
            pass
    
    
    def __del__(self):
        # close the pdf file
        if self.pdf_file:
            self.pdf_file.close()
    
    
    ###
    ### Extracting Images
    ###
    def write_file (self, folder, filename, filedata, flags='w'):
        """Write the file data to the folder and filename combination
        (flags: 'w' for write text, 'wb' for write binary, use 'a' instead of 'w' for append)"""
        result = False
        if os.path.isdir(folder):
            try:
                file_obj = open(os.path.join(folder, filename), flags)
                file_obj.write(filedata)
                file_obj.close()
                result = True
            except IOError:
                pass
        return result
    
    
    ###
    ### Extracting Text
    ###
    def to_bytestring (self, s, enc='utf-8'):
        """Convert the given unicode string to a bytestring, using the standard encoding,
        unless it's already a bytestring"""
        if s:
            if isinstance(s, str):
                return s
            else:
                return s.encode(enc)
    
    
    def update_page_text_hash (self, h, lt_obj, pct=0.2):
        """Use the bbox x0,x1 values within pct% to produce lists of associated text within the hash"""
    
        x0 = lt_obj.bbox[0]
        x1 = lt_obj.bbox[2]
    
        key_found = False
        for k, v in h.items():
            hash_x0 = k[0]
            if x0 >= (hash_x0 * (1.0-pct)) and (hash_x0 * (1.0+pct)) >= x0:
                hash_x1 = k[1]
                if x1 >= (hash_x1 * (1.0-pct)) and (hash_x1 * (1.0+pct)) >= x1:
                    # the text inside this LT* object was positioned at the same
                    # width as a prior series of text, so it belongs together
                    key_found = True
                    v.append(self.to_bytestring(lt_obj.get_text()))
                    h[k] = v
        if not key_found:
            # the text, based on width, is a new series,
            # so it gets its own series (entry in the hash)
            h[(x0,x1)] = [self.to_bytestring(lt_obj.get_text())]
    
        return h
    
    
    def _parse_lt_objs (self, lt_objs, page_number, text=[]):
        """Iterate through the list of LT* objects and capture the text or image data contained in each"""
        text_content = [] 
    
        page_text = {} # k=(x0, x1) of the bbox, v=list of text strings within that bbox width (physical column)
        for lt_obj in lt_objs:
            if isinstance(lt_obj, LTTextBox) or isinstance(lt_obj, LTTextLine):
                # text, so arrange is logically based on its column width
                page_text = self.update_page_text_hash(page_text, lt_obj)
            elif isinstance(lt_obj, LTFigure):
                # LTFigure objects are containers for other LT* objects, so recurse through the children
                text_content.append(self._parse_lt_objs(lt_obj, page_number, text_content))
    
        for k, v in sorted([(key,value) for (key,value) in page_text.items()]):
            # sort the page_text hash by the keys (x0,x1 values of the bbox),
            # which produces a top-down, left-to-right sequence of related columns
            text_content.append(''.join(v))
    
        return '\n'.join(text_content)
    

    def _parse_pages (self, doc):
        """With an open PDFDocument object, get the pages and parse each one"""
        rsrcmgr = PDFResourceManager()
        laparams = LAParams()
        device = PDFPageAggregator(rsrcmgr, laparams=laparams)
        interpreter = PDFPageInterpreter(rsrcmgr, device)
    
        text_content = []
        for i, page in enumerate(doc.get_pages()):
            interpreter.process_page(page)
            # receive the LTPage object for this page
            layout = device.get_result()
            # layout is an LTPage object which may contain child objects like LTTextBox, LTFigure, LTImage, etc.
            text_content.append(self._parse_lt_objs(layout, (i+1)))
    
        return text_content

    
    def create_pages(self):
        """Apply parsing function, returning the results"""

        from public_project.models import Page
        # create a parser object associated with the file object
        parser = PDFParser(self.pdf_file)
        # create a PDFDocument object that stores the document structure
        doc = PDFDocument()
        # connect the parser and document objects
        parser.set_document(doc)
        doc.set_parser(parser)
        # supply the password for initialization
        pdf_pwd = ''
        doc.initialize(pdf_pwd)

        if doc.is_extractable:
            # apply the function and return the result
            doc_pages = self._parse_pages(doc)

        i = 1
        for doc_page in doc_pages:
            page = Page(
                document=self.document,
                number=i,
                content = smart_unicode(doc_page, encoding='utf-8', strings_only=False, errors='strict'),
            )
            page.save()
            i = i + 1
        



from public_project.models import Document, Page, SearchTag, SearchTagCacheEntry
from public_project.search import get_query



def create_cache_entry(tag, document):
    entry_query = get_query(tag.name, ['document__title', 'content',])
    num_results = Page.objects.select_related().filter(document=document).filter(entry_query).count()
    
    if num_results > 0:
        ce = SearchTagCacheEntry()
        if SearchTagCacheEntry.objects.filter(document=document).filter(tag__name=tag.name).count() == 0 and \
           SearchTagCacheEntry.objects.filter(document=document).filter(tag__name=tag.name.title()).count() == 0 and \
           SearchTagCacheEntry.objects.filter(document=document).filter(tag__name=tag.name.upper()).count() == 0:
            ce.tag = tag
            ce.document = document
            ce.num_results = num_results
            ce.save()


def rebuild_cache_for_tag(tag):
    tag.searchtagcacheentry_set.all().delete()
    docs = Document.objects.all()
    for doc in docs:
        create_cache_entry(tag, doc)

def rebuild_cache_for_object(object):
    for tag in object.search_tags.all():
        rebuild_cache_for_tag(tag)


def rebuild_cache_for_document(document):
    SearchTagCacheEntry.objects.filter(document=document).delete()
    
    for tag in SearchTag.objects.all():
        create_cache_entry(tag, document)























from django.core.management.base import BaseCommand, CommandError
from ep_setup_app.example_data import ExampleData

class Command(BaseCommand):
    help = 'Creates the example data for the example project'

    def handle(self, *args, **options):
        ed = ExampleData()
        ed.create()



from tastypie import fields
from tastypie.resources import ModelResource
from public_project.models import ProjectPart, Question, Participant, Event, Page, Document


class ProjectPartsResource(ModelResource):
    
    class Meta:
        queryset = ProjectPart.objects.all()
        resource_name = 'project_parts'
        excludes = ['comments',]
        allowed_methods = ['get',]


class QuestionsResource(ModelResource):
    
    class Meta:
        queryset = Question.objects.all()
        resource_name = 'questions'
        excludes = ['comments',]
        allowed_methods = ['get',]


class ParticipantsResource(ModelResource):
    
    class Meta:
        queryset = Participant.objects.all()
        resource_name = 'participants'
        excludes = ['comments',]
        allowed_methods = ['get',]


class EventsResource(ModelResource):
    
    class Meta:
        queryset = Event.objects.all()
        resource_name = 'events'
        excludes = ['comments',]
        allowed_methods = ['get',]


class PagesResource(ModelResource):
    
    class Meta:
        queryset = Page.objects.all()
        resource_name = 'pages'
        allowed_methods = ['get',]
        

class DocumentsResource(ModelResource):
    pages = fields.ToManyField(PagesResource, 'page_set', null=True, full=True)
    
    class Meta:
        queryset = Document.objects.all()
        limit = 5
        resource_name = 'documents'
        excludes = ['pdf_images_generated', 'document', 'comments',]
        allowed_methods = ['get',]



from django.conf import settings
from django.conf.urls import patterns, include, url

from django.contrib import admin
admin.autodiscover()

from public_project.urls import urlpatterns

urlpatterns += patterns('',
    url(r'^admin/', include(admin.site.urls)),
)


if settings.DEBUG:
    urlpatterns += patterns('',
        url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {
            'document_root': settings.MEDIA_ROOT,
        }),
    )


from django.core.management.base import BaseCommand
from public_project.models import SearchTag
from public_project.tag_cache_creator import rebuild_cache_for_tag


class Command(BaseCommand):
    
    args = ''
    help = 'Rebuilding of all search tag cache entries for the project'
    
    def handle(self, *args, **options):
        tags = SearchTag.objects.all()
        for tag in tags:
            print tag
            rebuild_cache_for_tag(tag)



# coding=UTF-8
import os
import shutil
import subprocess
from datetime import datetime
from itertools import chain

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.contrib.sites.models import Site
from django.db import models
from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver
from django.utils.translation import ugettext, ugettext_lazy as _
from django.utils.http import urlquote
from public_project.doc_scanner import DocScanner



class UserProfile(models.Model):
    user = models.OneToOneField(User)
    receive_new_comment_emails = models.BooleanField(default=True)


def create_user_profile(sender, instance, created, **kwargs):
    if created:
        UserProfile.objects.create(user=instance)

post_save.connect(create_user_profile, sender=User)


class Image(models.Model):
    title = models.CharField(_("Title"), max_length=250)
    help_text = _("400px max width, 200px for images used with floating texts")
    image = models.ImageField(_("Image"), upload_to='images', help_text=help_text)
    help_text = _("Attribution to the original image source or alternatively something like 'Own image'.")
    attribution = models.CharField(_("Attribution"), max_length=250, help_text=help_text)
    help_text = _("Link to the original image source (if available)")
    attribution_url = models.URLField(_("Attribution URL"), help_text=help_text, blank=True, null=True)
    comments = models.TextField(_("Comments (internal)"), blank=True)
    
    def __unicode__(self):
        return self.title
    
    class Meta:
        ordering = ['title',]
        verbose_name = _('Image')
        verbose_name_plural = _('Images')


class SiteConfigManager(models.Manager):
    
    def get_site_config(self, request):
        if super(SiteConfigManager, self).count() == 0:
            site_config = SiteConfig()
            site_config.save()
        else:
            site_config = super(SiteConfigManager, self).all()[0]
        
        site_config.pdf_viewer = 'STANDARD'
        site_config.browser = 'Unknown'
        if request and 'HTTP_USER_AGENT' in request.META:
            if 'Mozilla'.lower() in request.META['HTTP_USER_AGENT'].lower():
                site_config.pdf_viewer = 'STANDARD'
                site_config.browser = 'Mozilla'
            if 'Safari'.lower() in request.META['HTTP_USER_AGENT'].lower():
                site_config.pdf_viewer = 'STANDARD'
                site_config.browser = 'Safari'
            if 'Chrome'.lower() in request.META['HTTP_USER_AGENT'].lower():
                site_config.pdf_viewer = 'STANDARD'
                site_config.browser = 'Chrome'
            if 'Opera'.lower() in request.META['HTTP_USER_AGENT'].lower():
                site_config.pdf_viewer = 'STANDARD'
                site_config.browser = 'Opera'
            if 'MSIE'.lower() in request.META['HTTP_USER_AGENT'].lower():
                if getattr(settings, 'DPP_IE_COMPATIBLE_PDF_VIEWER', False):
                    site_config.pdf_viewer = 'LEGACY'
                else:
                    site_config.pdf_viewer = False
                site_config.browser = 'MSIE'
        
        if getattr(settings, 'DPP_PUBLIC_API', False):
            site_config.public_api = True
        else:
            site_config.public_api = False
        
        return site_config
    

class SiteConfig(models.Model):
    help_text = _("Main title, shown in the header navi.")
    default = _("Project Website Title")
    title = models.CharField(_("Title"), max_length=250, help_text=help_text, default=default)
    help_text = _("Short version of the title, used e.g. in emails.")
    default = _("Project Website Short Title")
    short_title = models.CharField(_("Short title"), max_length=250, help_text=help_text, default=default)
    help_text = _("Color for the page title (Format: '#990000').")
    title_color = models.CharField(_("Title color"), max_length=7, help_text=help_text, default='#990000')
    help_text = _("Short intro text to describe your page (HTML possible), not too long, use about text for detailed information.")
    default = _("This is a project watch website.")
    intro_text = models.TextField(_("Intro text"), help_text=help_text, default=default)
    help_text = _("Short intro text about this site, what is the purpose, who is running it.")
    default = _("About text")
    about_text = models.TextField(_("About text"), help_text=help_text, default=default)
    help_text = _("Some html text you want to use in the footer of the page, you can e.g. \
provide a link to your email adress or associated social media sites.")
    footer = models.TextField(_("Footer"), help_text=help_text, default=_("Footer HTML Default"))
    help_text = _("Html to be displayed on the contact page, provide at least an adress there \
and some contact information.")
    contact_text = models.TextField(_("Contact"), help_text=help_text, default=_("Contact HTML Default"))
    comments = models.TextField(_("Comments (internal)"), blank=True)
    
    objects = SiteConfigManager()
    
    def __unicode__(self):
        return self.title
    
    class Meta:
        verbose_name = _('Website Configuration')
        verbose_name_plural = _('Website Configuration')


class SiteCategory(models.Model):
    NAME_CHOICES = (
        ('home', "Home"),
        ('project_parts', _('Topics')),
        ('goals', _('Goals')),
        ('questions', _('Questions')),
        ('participants', _('Participants')),
        ('events', _('Events')),
        ('documents', _('Documents')),
    )
    category = models.CharField(_("Category"), max_length=50, choices=NAME_CHOICES, unique=True)
    intro_text = models.TextField(_("Intro text"), blank=True, null=True)
    documents = models.ManyToManyField('Document', related_name="related_site_categories", blank=True, null=True, verbose_name=_("Documents"))
    web_sources = generic.GenericRelation('WebSource', verbose_name=_("Web Sources"))
    comments = models.TextField(_("Comments (internal)"), blank=True)
    date_added = models.DateTimeField(auto_now_add=True)
    
    def __unicode__(self):
        return self.get_category_display()
    
    class Meta:
        verbose_name = _('Website Category')
        verbose_name_plural = _('Website Categories')


class ParticipantType(models.Model):
    help_text = _("Type or category for sorting of participants")
    name = models.CharField(_("Title"), max_length=250, help_text=help_text)
    help_text = _("Use integer numbers for ordering (e.g. '100', '200', '300').")
    order = models.IntegerField(_("Order"), default=100)
    date_added = models.DateTimeField(auto_now_add=True)    
    
    def get_participants(self):
        return self.participant_set.filter(belongs_to=None)
    
    def __unicode__(self):
        return self.name
    
    class Meta:
        ordering = ['order']
        verbose_name = _('Type Participants')
        verbose_name_plural = _('Types Participants')


class WebSource(models.Model):
    title = models.CharField(_("Title"), max_length=250)
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey("content_type", "object_id")
    help_text = _("Use integer numbers for ordering (e.g. '100', '200', '300').")
    order = models.IntegerField(_("Order"), help_text=help_text, default=100, blank=True, null=True)
    url = models.URLField(_("URL"))
    date = models.DateField(_("Date"), blank=True, null=True)
    date_added = models.DateTimeField(auto_now_add=True)
    
    def __unicode__(self):
        return self.title
    
    def get_absolute_url(self):
        return "/" + ugettext("web_sources_url") + unicode(self.id) + "/"
    
    @classmethod
    def get_icon_class(cls):
        return 'icon-globe'
    
    class Meta:
        ordering = ['order']
        verbose_name = _('Web-Source')
        verbose_name_plural = _('Web-Sources')


class Membership(models.Model):
    from_participant = models.ForeignKey('Participant', related_name='from_memberships', verbose_name=_("From participant"))
    help_text = _('Association with a group or institution')
    to_participant = models.ForeignKey('Participant', help_text=help_text, related_name='to_memberships', verbose_name=_("To participant"))
    help_text = _("Type or function of the membership or task or position of the participant.")
    function = models.CharField(_("Function"), max_length=50, help_text=help_text, blank=True, null=True)
    active = models.BooleanField(_("Active"), default=True)
    
    class Meta:
        verbose_name = _('Membership')
        verbose_name_plural = _('Memberships')


class Participant(models.Model):
    help_text  = _("Person, group or institution acting in some way in the context of the project or being affected by the process or the result of the project execution.")
    name = models.CharField(_("Name"), max_length=250, help_text=help_text, unique=True)
    help_text = _("Type for sorting, only for groups/institutions, necessary for participant to be visible on site")
    type = models.ForeignKey(ParticipantType, blank=True, null=True, help_text=help_text)
    help_text = _("Use integer numbers for ordering (e.g. '100', '200', '300').")
    order = models.IntegerField(_("Order"), help_text=help_text, default=500, blank=True, null=True)
    help_text = _("The participant belongs to another participant (often an institution or group), leave blank if participant itself is institution/group.")
    belongs_to = models.ManyToManyField('self', symmetrical=False, through='Membership', verbose_name=_("Belongs to"))
    search_tags = generic.GenericRelation('SearchTag')
    help_text = _("Role/tasks as well as interests/goals of the participant regarding the project.")
    description = models.TextField(_("Description"), help_text=help_text)
    web_sources = generic.GenericRelation(WebSource)
    comment_relations = generic.GenericRelation('CommentRelation')
    comments = models.TextField(_("Comments (internal)"), blank=True)
    date_added = models.DateTimeField(auto_now_add=True)
    activities = generic.GenericRelation('ActivityLog')
    
    def __unicode__(self):
        return self.name
    
    def get_questions(self):
        return Question.objects.filter(participants__in=list(chain([self,], self.participant_set.all()))).distinct()
    
    def get_events(self):
        return Event.objects.filter(participants__in=list(chain([self,], self.participant_set.all()))).distinct()
    
    def get_documents(self):
        return Document.objects.filter(participants__in=list(chain([self,], self.participant_set.all()))).distinct()
    
    def get_feed_description(self):
        html  = self.description
        return  html
    
    def get_absolute_url(self):
        return "/" + ugettext("participants_url") + unicode(self.id) + "/"
    
    def get_comments_url(self):
        return "/" + ugettext("participants_url") + unicode(self.id) + "/" + ugettext("comments_url")
    
    def get_active_user_comments(self):
        return self.comment_relations.filter(comment__published=True)
    
    def get_num_active_user_comments(self):
        return self.comment_relations.filter(comment__published=True).count()
    
    @classmethod
    def get_color(cls):
        return '#3e3ec7';
    
    def get_icon_class(self):
        if self.belongs_to.count() > 0:
            return 'icon-user'
        else:
            return 'icon-group'
    
    def get_simple_entry(self):
        html  = '<i class="' + self.get_icon_class() + '"></i> '
        html += '<a href="' + self.get_absolute_url() +'">' + self.name + '</a>'
        return html 
    
    class Meta:
        ordering = ['type', 'order', 'name',]
        verbose_name = _('Participant')
        verbose_name_plural = _('Participants')


class ProjectPart(models.Model):
    help_text = _("Name of the topic")
    name = models.CharField(_("Name"), max_length=250, help_text=help_text)
    help_text = _("Use integer numbers for ordering (e.g. '100', '200', '300').")
    order = models.IntegerField(_("Order"), help_text=help_text, default=500, blank=True, null=True)
    help_text = _("If you select another project part here, you'll make this a sub project part.")
    main_project_parts = models.ManyToManyField('self', symmetrical=False, help_text=help_text, blank=True, null=True, verbose_name=_("Main Topic"))
    search_tags = generic.GenericRelation('SearchTag')
    help_text = _("Website (if existant).")
    description = models.TextField(_("Description"), help_text=help_text)
    web_sources = generic.GenericRelation(WebSource)
    comment_relations = generic.GenericRelation('CommentRelation')
    comments = models.TextField(_("Comments (internal)"), blank=True)
    date_added = models.DateTimeField(auto_now_add=True)
    activities = generic.GenericRelation('ActivityLog')

    def __unicode__(self):
        return self.name
    
    def get_questions(self):
        return Question.objects.filter(project_parts__in=list(chain([self,], self.projectpart_set.all()))).distinct()
    
    def get_num_questions(self):
        return Question.objects.filter(project_parts__in=list(chain([self,], self.projectpart_set.all()))).distinct().count()
    
    def get_events(self):
        return Event.objects.filter(project_parts__in=list(chain([self,], self.projectpart_set.all()))).distinct()
    
    def get_num_events(self):
        return Event.objects.filter(project_parts__in=list(chain([self,], self.projectpart_set.all()))).distinct().count()
    
    def get_documents(self):
        return Document.objects.filter(project_parts__in=list(chain([self,], self.projectpart_set.all()))).distinct()
    
    def get_num_documents(self):
        return Document.objects.filter(project_parts__in=list(chain([self,], self.projectpart_set.all()))).distinct().count()
    
    def get_feed_description(self):
        html  = self.description
        return  html
    
    def get_absolute_url(self):
        return "/" + ugettext("project_parts_url") + unicode(self.id) + "/"
    
    @classmethod
    def get_color(cls):
        return '#0d9434';
    
    def get_icon_class(self):
        if self.main_project_parts.count() > 0:
            return 'icon-cog'
        else:
            return 'icon-cogs'
    
    def get_simple_entry(self):
        html  = '<i class="' + self.get_icon_class() + '"></i> '
        html += '<a href="' + self.get_absolute_url() +'">' + self.name + '</a>'
        return html
    
    class Meta:
        ordering = ['order', 'name',]
        verbose_name = _('Project Part')
        verbose_name_plural = _('Project Parts')


class Event(models.Model):
    EVENT_TYPE_CHOICES = (
        ('ME', _('Meeting / Gathering / Session')),
        ('IN', _('New Information / Decision / Statement')),
        ('MI', _('Project Progress / Execution / Milestone')),
        ('CI', _('Action by Civil Society')),
        ('UE', _('Unplanned Event')),
        ('SE', _('Miscellaneous')),
    )
    EVENT_TYPE_CHOICES_ICONS = {
        'ME': 'icon-calendar',
        'IN': 'icon-info-sign',
        'MI': 'icon-wrench',
        'CI': 'icon-bullhorn',
        'UE': 'icon-bolt',
        'SE': 'icon-asterisk',
    }
    title = models.CharField(_("Title"), max_length=250)
    event_type = models.CharField(_("Type"), max_length=2, choices=EVENT_TYPE_CHOICES)
    help_text = _("Event being of central importance for the project.")
    important = models.BooleanField(_("Main Event"), help_text=help_text)
    search_tags = generic.GenericRelation('SearchTag')
    description = models.TextField(_("Description"))
    date = models.DateField(_("Date"))
    help_text = _("Date is not exact (e.g. if a source refers only to the month)")
    not_exact = models.BooleanField(_("Date not exact"), default=False, help_text=help_text)
    participants = models.ManyToManyField(Participant, related_name="related_events", blank=True, null=True, verbose_name=_("Participants"))
    project_parts = models.ManyToManyField(ProjectPart, related_name="related_events", blank=True, null=True, verbose_name=_("Topics"))
    web_sources = generic.GenericRelation(WebSource)
    comment_relations = generic.GenericRelation('CommentRelation')
    comments = models.TextField(_("Comments (internal)"), blank=True)
    date_added = models.DateTimeField(auto_now_add=True)
    activities = generic.GenericRelation('ActivityLog')
    
    def __unicode__(self):
        str  = self.title + ", "
        if self.not_exact:
            str += '('
        str += datetime.strftime(self.date, '%d.%m.%Y')
        if self.not_exact:
            str += ')'
        return str 
    
    def get_feed_description(self):
        html  = self.description
        
        if len(self.participants.all()) > 0:
            html += '<br><br>'
            html += _("Participants") + ':'
            for p in self.participants.all():
                html += '<br><a href="' + p.get_absolute_url() + '">' + unicode(p) + '</a>'
        
        if len(self.project_parts.all()) > 0:
            html += '<br><br>'
            html += _("Subject Areas") + ':'
            for pp in self.project_parts.all():
                html += '<br><a href="' + pp.get_absolute_url() + '">' + unicode(pp) + '</a>'
        
        return  html
    
    def get_absolute_url(self):
        return "/" + ugettext("events_url") + unicode(self.id) + "/"
    
    def get_event_type_icon(self):
        return self.EVENT_TYPE_CHOICES_ICONS[self.event_type]
    
    def get_comments_url(self):
        return "/" + ugettext("events_url") + unicode(self.id) + "/" + ugettext("comments_url")
    
    def get_active_user_comments(self):
        return self.comment_relations.filter(comment__published=True)
    
    def get_num_active_user_comments(self):
        return self.comment_relations.filter(comment__published=True).count()
    
    @classmethod
    def get_color(cls):
        return '#c91a1a'
    
    @classmethod
    def get_icon_class(cls):
        return 'icon-time'
    
    def get_simple_entry(self):
        html  = '<i class="' + self.get_icon_class() + '"></i> '
        html += '<a href="' + self.get_absolute_url() +'">' + self.title + '</a>'
        return html
    
    def as_list(self):
        return [self,]
    
    class Meta:
        ordering = ['-date']
        verbose_name = _('Event')
        verbose_name_plural = _('Events')


class Question(models.Model):
    help_text = _("Title/short version of the question. Use prefix (e.g. 1,2,3 or A1,A2,A3) to sort questions")
    title = models.CharField(_("Title"), max_length=250, help_text=help_text)
    answered = models.BooleanField(_("Answered"))
    help_text = _("Description/long version of the question.")
    description = models.TextField(_("Description"), help_text=help_text)
    project_parts = models.ManyToManyField(ProjectPart, related_name="related_questions", blank=True, null=True, verbose_name=_("Topics"))
    participants = models.ManyToManyField(Participant, related_name="related_questions", blank=True, null=True, verbose_name=_("Participants"))
    events = models.ManyToManyField(Event, related_name="related_questions", blank=True, null=True, verbose_name=_("Events"))
    help_text = _("Optional explanations or remarks around the question")
    explanations = models.TextField(_("Explanations"), blank=True, help_text=help_text)
    help_text = _("Optional answer (summary) of a question")
    answer = models.TextField(_("Answer"), blank=True, help_text=help_text)
    documents = models.ManyToManyField('Document', related_name="related_documents", blank=True, null=True, verbose_name=_("Documents"))
    web_sources = generic.GenericRelation(WebSource)
    comment_relations = generic.GenericRelation('CommentRelation')
    comments = models.TextField(_("Comments (internal)"), blank=True)
    date_added = models.DateTimeField(auto_now_add=True)
    activities = generic.GenericRelation('ActivityLog')
    
    def get_feed_description(self):
        html  = self.description
        
        if len(self.project_parts.all()) > 0:
            html += '<br><br>'
            html += _("Subject Areas") + ':'
            for pp in self.project_parts.all():
                html += '<br><a href="' + pp.get_absolute_url() + '">' + unicode(pp) + '</a>'
        
        if len(self.participants.all()) > 0:
            html += '<br><br>'
            html += _("Participants") + ':'
            for p in self.participants.all():
                html += '<br><a href="' + p.get_absolute_url() + '">' + unicode(p) + '</a>'
        
        if len(self.events.all()) > 0:
            html += '<br><br>'
            html += _("Events") + ':'
            for e in self.events.all():
                html += '<br><a href="' + e.get_absolute_url() + '">' + unicode(e) + '</a>'
        
        return  html
    
    def get_absolute_url(self):
        return "/" + ugettext("questions_url") + unicode(self.id) + "/"
    
    def get_comments_url(self):
        return "/" + ugettext("questions_url") + unicode(self.id) + "/" + ugettext("comments_url")
    
    def get_active_user_comments(self):
        return self.comment_relations.filter(comment__published=True)
    
    def get_num_active_user_comments(self):
        return self.comment_relations.filter(comment__published=True).count()
    
    @classmethod
    def get_color(cls):
        return '#941bbf';
    
    @classmethod
    def get_icon_class(cls):
        return 'icon-question-sign';
    
    def __unicode__(self):
        return self.title
    
    class Meta:
        ordering = ['title']
        verbose_name = _('Question')
        verbose_name_plural = _('Questions')


class ProjectGoalGroupManager(models.Manager):
    
    def get_current(self):
        if self.count() > 0:
            return self.all().order_by('event')[0]
        else:
            return None
        

class ProjectGoalGroup(models.Model):
    help_text = _("Group of project goals being determined at a certain point in time.")
    title = models.CharField(_("Title"), max_length=250, help_text=help_text)
    help_text = _("The topic the group of goals belong to. Will be shown as common goal group if none selected.")
    project_part = models.ForeignKey(ProjectPart, blank=True, null=True, verbose_name=_("Topic"))
    event = models.ForeignKey(Event, verbose_name=_("Associated event"))
    is_current = models.BooleanField(default=True, verbose_name=_("Is current"))
    help_text = _("Description of the group of project goals.")
    description = models.TextField(_("Description"), help_text=help_text)
    comments = models.TextField(_("Comments (internal)"), blank=True)
    objects = ProjectGoalGroupManager()
    date_added = models.DateTimeField(auto_now_add=True)
    activities = generic.GenericRelation('ActivityLog')
    
    def __unicode__(self):
        return self.title
    
    class Meta:
        verbose_name = _('Goal')
        verbose_name_plural = _('Goals')


class ProjectGoal(models.Model):
    help_text = _("Name, e.g. 'Project budget', 'Target date', 'Noise level'")
    name = models.CharField(_("Name"), max_length=250, help_text=help_text)
    project_goal_group = models.ForeignKey(ProjectGoalGroup)
    help_text = _("Single performance figure describing the project goal, e.g. '1.000.000 Euro', 'January 25th 2020', ...")
    performance_figure = models.CharField(_("Performance figure"), max_length=250, help_text=help_text)
    help_text = _("Use integer numbers for ordering (e.g. '100', '200', '300').")
    order = models.IntegerField(_("Order"), help_text=help_text, default=100, blank=True, null=True)
    
    def __unicode__(self):
        return self.name
    
    class Meta:
        ordering = ['order',]
        verbose_name = _('Goal Value')
        verbose_name_plural = _('Goal Values')


class Document(models.Model):
    help_text = _("Title of the document")
    title = models.CharField(_("Title"), max_length=250, help_text=help_text)
    help_text = _('Document in pdf format')
    document = models.FileField(_("Document"), upload_to='documents', help_text=help_text)
    help_text = _('Date of creation of the document')
    date = models.DateField(_("Date"), help_text=help_text)
    help_text = _("Short description.")
    description = models.TextField(_("Description"), help_text=help_text)
    participants = models.ManyToManyField(Participant, related_name="related_documents", blank=True, null=True, verbose_name=_("Participants"))
    project_parts = models.ManyToManyField(ProjectPart, related_name="related_documents", blank=True, null=True, verbose_name=_("Topics"))
    events = models.ManyToManyField(Event, related_name="related_documents", blank=True, null=True, verbose_name=_("Events"))
    comment_relations = generic.GenericRelation('CommentRelation')
    comments = models.TextField(_("Comments (internal)"), blank=True)
    pdf_images_generated = models.BooleanField(default=False)
    date_added = models.DateTimeField(auto_now_add=True)
    activities = generic.GenericRelation('ActivityLog')
    
    def __unicode__(self):
        return self.title + " (" + datetime.strftime(self.date, '%d.%m.%Y') + ")"
    
    def get_feed_description(self):
        html  = self.description
        
        if len(self.participants.all()) > 0:
            html += '<br><br>'
            html += _("Participants") + ':'
            for p in self.participants.all():
                html += '<br><a href="' + p.get_absolute_url() + '">' + unicode(p) + '</a>'
        
        if len(self.project_parts.all()) > 0:
            html += '<br><br>'
            html += _("Subject Areas") + ':'
            for pp in self.project_parts.all():
                html += '<br><a href="' + pp.get_absolute_url() + '">' + unicode(pp) + '</a>'
        
        if len(self.events.all()) > 0:
            html += '<br><br>'
            html += _("Events") + ':'
            for e in self.events.all():
                html += '<br><a href="' + e.get_absolute_url() + '">' + unicode(e) + '</a>'
        
        return  html
    
    def get_absolute_url(self):
        return "/" + ugettext("documents_url") + unicode(self.id) + "/"
    
    def get_comments_url(self):
        return "/" + ugettext("documents_url") + unicode(self.id) + "/" + ugettext("comments_url")
    
    @classmethod
    def get_color(cls):
        return '#b88f00';
    
    @classmethod
    def get_icon_class(cls):
        return 'icon-inbox';
    
    def get_document_name(self):
        return os.path.basename(self.document.name)
    
    def get_pages_path(self):
        return os.path.join(settings.MEDIA_ROOT, 'documents/document_' + unicode(self.id) + '/')
    
    def get_active_user_comments(self):
        return self.comment_relations.filter(comment__published=True)
    
    def get_num_active_user_comments(self):
        return self.comment_relations.filter(comment__published=True).count()
    
    class Meta:
        ordering = ['-date_added']
        verbose_name = _('Document')
        verbose_name_plural = _('Documents')
    
    def __init__(self, *args, **kwargs):
        super(Document, self).__init__(*args, **kwargs)
        self.old_document = self.document

    def save(self, force_insert=False, force_update=False):
        if getattr(settings, 'DPP_IE_COMPATIBLE_PDF_VIEWER', True) and self.old_document != self.document:
            self.pdf_images_generated = False
        else:
            self.pdf_images_generated = True
        super(Document, self).save(force_insert, force_update)
        #print "pdf_images_generated set to: " + str(self.pdf_images_generated)

        # Delete old document
        if self.old_document and self.old_document != self.document:
            if os.path.exists(self.old_document.path):
                os.remove(self.old_document.path)
                #print "Old document deleted from path: " + self.old_document.path
        
        if self.old_document != self.document:
            cmd = u"python manage.py createpages " + str(self.id) + " --settings=" + settings.SETTINGS_MODULE
            subprocess.Popen(cmd, shell=True)
            #print "New page creation process started..."
        
        # Creating images when DPP_IE_COMPATIBLE_PDF_VIEWER=True in settings.py    
        if getattr(settings, 'DPP_IE_COMPATIBLE_PDF_VIEWER', True) and self.old_document != self.document:
            cmd = u"python manage.py generatepdfimages " + str(self.id) + " --settings=" + settings.SETTINGS_MODULE
            subprocess.Popen(cmd, shell=True)
            #print "Image generation process started..."
        
        self.old_document = self.document


def delete_pages_folder(sender, **kwargs):
    instance = kwargs['instance']
    if os.path.exists(instance.get_pages_path()):
        shutil.rmtree(instance.get_pages_path())

def delete_document_file(sender, **kwargs):
    instance = kwargs['instance']
    if instance.document and os.path.exists(instance.document.path):
                os.remove(instance.document.path)

pre_delete.connect(delete_pages_folder, sender=Document)
pre_delete.connect(delete_document_file, sender=Document)


class Page(models.Model):
    document = models.ForeignKey(Document)
    number = models.IntegerField()
    content = models.TextField(blank=True)
    
    def get_filename(self):
        return u'page-' + unicode(self.number) + u'.jpg'
    
    def get_filepath(self):
        return self.document.get_pages_path() + self.get_filename()
    
    def __unicode__(self):
        return unicode(self.document) + ", Page " + unicode(self.number)
    
    class Meta:
        ordering = ['number']
        verbose_name = _('Page')
        verbose_name_plural = _('Pages')


def delete_page_image(sender, **kwargs):
    instance = kwargs['instance']
    if os.path.exists(instance.get_filepath()):
        os.remove(instance.get_filepath())

pre_delete.connect(delete_page_image, sender=Page)


class SearchTag(models.Model):
    help_text = _("Documents containing these search tags are shown on the detail page of this object.")
    name = models.CharField(_("Name"), max_length=250, help_text=help_text)
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey("content_type", "object_id")
    order = models.IntegerField(_("Order"), default=100, blank=True, null=True)
    date_added = models.DateTimeField(auto_now_add=True)
    
    def __unicode__(self):
        return self.name
    
    class Meta:
        ordering = ['order']
        verbose_name = _('Search Tag')
        verbose_name_plural = _('Search Tags')


@receiver(post_save, sender=SearchTag)
def rebuild_tag_cache(sender, instance, **kwargs):
    from public_project.tag_cache_creator import rebuild_cache_for_tag
    rebuild_cache_for_tag(instance)


class SearchTagCacheEntry(models.Model):
    tag = models.ForeignKey(SearchTag)
    document = models.ForeignKey(Document)
    num_results = models.IntegerField()
    
    def __unicode__(self):
        return "Tag:" + unicode(self.tag) + ", Document:" + unicode(self.document)
    
    class Meta:
        ordering = ['-num_results']


RELATION_LIMIT = models.Q(app_label = 'public_project', model = 'projectpart') | \
            models.Q(app_label = 'public_project', model = 'question') | \
            models.Q(app_label = 'public_project', model = 'participant') | \
            models.Q(app_label = 'public_project', model = 'event') | \
            models.Q(app_label = 'public_project', model = 'document')


class ResearchRequestRelation(models.Model):
    research_request = models.ForeignKey('ResearchRequest')
    help_text = _('Type of the related object (ProjectPart, Question, Participant, Event, Document).')
    limit = RELATION_LIMIT
    content_type = models.ForeignKey(ContentType, help_text=help_text, limit_choices_to=limit)
    help_text = _('The id of the related object (you can find the id of an object in the url \
of the object change form in the admin).')
    object_id = models.PositiveIntegerField(help_text=help_text)
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    help_text = _("Page number in document")
    page = models.IntegerField(_("Page"), blank=True, null=True, help_text=help_text)
    
    def __unicode__(self):
        return unicode(self.research_request) + ', ' + unicode(self.content_object)
    
    class Meta:
        verbose_name = _('Relation with Project Element')
        verbose_name_plural = _('Relations with Project Elements')
    

class ResearchRequest(models.Model):
    help_text = _('Give a unique number to your request so that people can reference it (e.g. "R1", "R2",...)')
    nr = models.CharField(_("Nr"), max_length=8, help_text=help_text)
    title = models.CharField(_("Title"), max_length=250)
    open = models.BooleanField(_("Open"), default=True)
    description = models.TextField(_("Description"))
    date_added = models.DateTimeField(auto_now_add=True)
    activities = generic.GenericRelation('ActivityLog')
    
    def __unicode__(self):
        return unicode(self.nr) + ': ' + self.title
    
    def get_feed_description(self):
        html  = self.description
        
        html += '<br><br>'
        html += _("Request on the following contents") + ':'
        for rr_rel in self.researchrequestrelation_set.all():
            html += '<br><a href="' + rr_rel.content_object.get_absolute_url() + '">' + unicode(rr_rel.content_object) + '</a>'
        
        return  html
    
    def get_related_question(self):
        question = None
        content_type = ContentType.objects.get(app_label="public_project", model="question")
        cos =  self.researchrequestrelation_set.all()
        for co in cos:
            if co.content_type == content_type:
                question = co.content_object
        return question
    
    def get_absolute_url(self):
        return self.get_related_question().get_absolute_url() + '?research_request_id=' + unicode(self.id)
    
    @classmethod
    def get_color(cls):
        return '#555';
    
    @classmethod
    def get_icon_class(cls):
        return 'icon-eye-open';
    
    def get_share_text(self):
        short_title = 'ProjectWatch'
        if SiteConfig.objects.count() > 0:
            site_config = SiteConfig.objects.all()[0]
            short_title = site_config.short_title
        
        share_text = short_title + " " + unicode(_("Research Request")) + " " + self.nr + ": " + self.title
        return share_text
    
    def get_share_url(self):
        share_url = 'http://%s%s' % (Site.objects.get_current().domain, self.get_absolute_url())
        return share_url
    
    def get_twitter_url(self):
        url  = 'https://twitter.com/intent/tweet'
        url += '?&text=' + urlquote(self.get_share_text()) + '&url=' + urlquote(self.get_share_url())
        return url
    
    def get_facebook_url(self):
        url  = 'https://www.facebook.com/sharer.php'
        url += '?&t=' + urlquote(self.get_share_text()) + '&u=' + urlquote(self.get_share_url())
        return url
    
    def get_google_plus_url(self):
        url  = 'https://plus.google.com/share'
        url += '?url=' + urlquote(self.get_share_url())
        return url
    
    def get_app_net_url(self):
        url  = 'https://alpha.app.net/intent/post?text='
        url += urlquote(self.get_share_text() + " " + self.get_share_url())
        return url
    
    class Meta:
        ordering = ['-date_added']
        verbose_name = _('Research Request')
        verbose_name_plural = _('Research Requests')



class CommentRelation(models.Model):
    comment = models.ForeignKey('Comment')
    help_text = _('Type of the related object (ProjectPart, Question, Participant, Event, Document).')
    limit = RELATION_LIMIT
    content_type = models.ForeignKey(ContentType, help_text=help_text, limit_choices_to=limit)
    help_text = _('The id of the related object (you can find the id of an object in the url \
of the object change form in the admin).')
    object_id = models.PositiveIntegerField(help_text=help_text)
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    help_text = _("Page number in document")
    page = models.IntegerField(_("Page"), blank=True, null=True, help_text=help_text)
    
    def __unicode__(self):
        return unicode(self.comment) + ', ' + unicode(self.content_object)
    
    class Meta:
        verbose_name = _('Relation with Project Element')
        verbose_name_plural = _('Relations with Project Elements')

class Comment(models.Model):
    username = models.CharField(_("Username"), max_length=250)
    email = models.EmailField(_("Email"), max_length=250)
    help_text = _('User has given permission to get in contact via email.')
    feedback_allowed = models.BooleanField(_("Feedback allowed"), default=False, help_text=help_text)
    comment = models.TextField(_("Comment text"))
    help_text = _('Comment is only shown on page if published is true.')
    published = models.BooleanField(_("Published"), default=False, help_text=help_text)
    published_by = models.CharField(_("Published by"), max_length=250, blank=True)
    activation_hash = models.CharField(_("Activation hash"), max_length=250, blank=True)
    date_added = models.DateTimeField(auto_now_add=True)
    activities = generic.GenericRelation('ActivityLog')
    
    def __unicode__(self):
        return self.username + ", " + datetime.strftime(self.date_added, '%d.%m.%Y')
    
    def get_feed_description(self):
        html  = self.comment
        
        html += '<br><br>'
        html += _("Commented contents") + ':'
        for cr in self.commentrelation_set.all():
            html += '<br><a href="' + cr.content_object.get_absolute_url() + '">' + unicode(cr.content_object) + '</a>'
        
        return  html
    
    def get_absolute_url(self):
        first_co = self.commentrelation_set.all()[0].content_object
        return first_co.get_absolute_url() + '?comment_id=' + unicode(self.id)
    
    @classmethod
    def get_color(cls):
        return '#555';
    
    @classmethod
    def get_icon_class(cls):
        return 'icon-comment';
    
    def get_share_text(self):
        co1 = list(self.commentrelation_set.all())[0].content_object
        
        short_title = 'ProjectWatch'
        if SiteConfig.objects.count() > 0:
            site_config = SiteConfig.objects.all()[0]
            short_title = site_config.short_title
        
        share_text = _('Comment on %s on') % short_title
        share_text += ' "' + unicode(co1) + '"'
        share_text += ":"
        return share_text
    
    def get_share_url(self):
        share_url = 'http://%s%s' % (Site.objects.get_current().domain, self.get_absolute_url())
        return share_url
    
    def get_twitter_url(self):
        url  = 'https://twitter.com/intent/tweet'
        url += '?&text=' + urlquote(self.get_share_text()) + '&url=' + urlquote(self.get_share_url())
        return url
    
    def get_facebook_url(self):
        url  = 'https://www.facebook.com/sharer.php'
        url += '?&t=' + urlquote(self.get_share_text()) + '&u=' + urlquote(self.get_share_url())
        return url
    
    def get_google_plus_url(self):
        url  = 'https://plus.google.com/share'
        url += '?url=' + urlquote(self.get_share_url())
        return url
    
    def get_app_net_url(self):
        url  = 'https://alpha.app.net/intent/post?text='
        url += urlquote(self.get_share_text() + " " + self.get_share_url())
        return url
    
    class Meta:
        ordering = ['-date_added']
        verbose_name = _('Comment')
        verbose_name_plural = _('Comments')
    

class ActivityLog(models.Model):
    TYPE_CHOICES = (
        ('NA', _('New Subject Area')),
        ('NQ', _('New Question')),
        ('PA', _('New Participant')),
        ('NE', _('Event')),
        ('ND', _('New Document')),
        ('RR', _('Research Request')),
        ('NC', _('Comment')),
    )
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    type = models.CharField(_("Type"), max_length=2, choices=TYPE_CHOICES)
    info = models.CharField(_("Info"), max_length=250, blank=True)
    date = models.DateTimeField(_("Date"), auto_now_add=True)
    
    def __unicode__(self):
        return u'Activity: ' + unicode(self.content_object)
    
    class Meta:
        ordering = ['-date']
        verbose_name = _('Activity Log Entry')
        verbose_name_plural = _('Activity Log')


@receiver(post_save, sender=ProjectPart)
def add_na_activity(sender, instance, created, **kwargs):
    if created:
        al = ActivityLog(content_object=instance, type='NA')
        al.save()


@receiver(post_save, sender=Question)
def add_nq_activity(sender, instance, created, **kwargs):
    if created:
        al = ActivityLog(content_object=instance, type='NQ')
        al.save()


@receiver(post_save, sender=Participant)
def add_pa_activity(sender, instance, created, **kwargs):
    if created:
        al = ActivityLog(content_object=instance, type='PA')
        al.save()


@receiver(post_save, sender=Event)
def add_ne_activity(sender, instance, created, **kwargs):
    if created:
        al = ActivityLog(content_object=instance, type='NE')
        al.save()


@receiver(post_save, sender=Document)
def add_nd_activity(sender, instance, created, **kwargs):
    if created:
        al = ActivityLog(content_object=instance, type='ND')
        al.save()


@receiver(post_save, sender=ResearchRequest)
def add_rr_activity(sender, instance, created, **kwargs):
    if created:
        al = ActivityLog(content_object=instance, type='RR')
        al.save()



from django.core.management.base import BaseCommand, CommandError
from public_project.models import Document
from public_project.doc_scanner import DocScanner
from public_project.tag_cache_creator import rebuild_cache_for_document

class Command(BaseCommand):
    
    args = '<document_id>'
    help = 'Used internally to generate pages from pdf documents'
    
    def handle(self, *args, **options):
        for document_id in args:
            try:
                document = Document.objects.get(pk=int(document_id))
            except Document.DoesNotExist:
                raise CommandError('Document "%s" does not exist' % document_id)
            document.page_set.all().delete()
            ds = DocScanner(document)
            ds.create_pages()
            
            rebuild_cache_for_document(document)



import os
import subprocess
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from public_project.models import Document

class Command(BaseCommand):
    
    args = '<document_id>'
    help = 'Used internally to generate images from pdf document pages'
    
    def handle(self, *args, **options):
        for document_id in args:
            try:
                document = Document.objects.get(pk=int(document_id))
            except Document.DoesNotExist:
                raise CommandError('Document "%s" does not exist' % document_id)
            if document.page_set.count() > 0:
                raise CommandError('Pages for document "%s" already created' % document_id)
            
            #Create images
            pdf_path = os.path.join(settings.MEDIA_ROOT, unicode(document.document))
            path = document.get_pages_path()
            if not os.path.exists(path):
                os.makedirs(path)
            
            cmd = u"convert -quality 90 -density 100 '"  + pdf_path + "' '" + path + "page-%d.png'"
            p = subprocess.Popen(cmd, shell=True)
            retval = p.wait()
            document.pdf_images_generated = True
            document.save()
            


# coding=UTF-8
import json, math, os, urllib, urllib2

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.mail import send_mail
from django.db.models import Count
from django.http import HttpResponse, Http404
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.utils.html import strip_tags
from django.utils.translation import ugettext as _
from public_project.forms import *
from public_project.models import *
from public_project.search import get_query, search_for_documents


def get_research_request(request):
    if 'research_request_id' in request.GET:
        try:
            research_request = ResearchRequest.objects.get(pk=request.GET['research_request_id'])
            return [research_request,]
        except ResearchRequest.DoesNotExist:
            pass
    return None


def get_user_comment(request):
    if 'comment_id' in request.GET:
        try:
            comment = Comment.objects.get(pk=request.GET['comment_id'])
            return [comment,]
        except Comment.DoesNotExist:
            pass
    return None
    

def validate_research_request_form(request):
    
    if request.method == 'POST' and 'research_request_form' in request.POST:
        if not request.user.has_perm('public_project.can_add_research_request'):
            return "FAILED"
        rr = ResearchRequest()
        rr.nr = request.POST['nr']
        rr.title = request.POST['title']
        rr.description = request.POST['description']
        rr.save()
            
        co1_content_type = ContentType.objects.get(app_label="public_project", model=request.POST['co1_content_type'])
        co1 = co1_content_type.get_object_for_this_type(id=request.POST['co1_id'])
        rr_rel = ResearchRequestRelation()
        rr_rel.research_request = rr
        rr_rel.content_object = co1
        if(request.POST['co1_content_type'] == 'document'):
            rr_rel.page = request.POST['co1_page']
        rr_rel.save()
            
        counter_more_cos = int(request.POST['counter_more_cos'])
            
        for num_id in range(2, counter_more_cos + 2):
            if 'co' + str(num_id) + '_id' in request.POST:
                co_content_type = ContentType.objects.get(app_label="public_project", model=request.POST['co' + str(num_id) + '_content_type'])
                co = co_content_type.get_object_for_this_type(id=request.POST['co' + str(num_id) + '_id'])
                rr_rel = ResearchRequestRelation()
                rr_rel.research_request = rr
                rr_rel.content_object = co
                if(request.POST['co' + str(num_id) + '_content_type'] == 'document'):
                    rr_rel.page = request.POST['co' + str(num_id) + '_page']
                rr_rel.save()        
        return "SENT"
    
    return ""


def validate_comment_form(request):
    
    if request.method == 'POST' and 'comment_form' in request.POST:
        print request.POST
        form = CommentForm(request.POST)
        if form.is_valid():
            c = Comment()
            c.username = form.cleaned_data['username']
            c.email = form.cleaned_data['email']
            c.comment = form.cleaned_data['comment']
            c.feedback_allowed = form.cleaned_data['feedback_allowed']
            c.activation_hash = os.urandom(16).encode('hex')
            c.save()
            
            co1_content_type = ContentType.objects.get(app_label="public_project", model=form.cleaned_data['co1_content_type'])
            co1 = co1_content_type.get_object_for_this_type(id=form.cleaned_data['co1_id'])
            cr = CommentRelation()
            cr.comment = c
            cr.content_object = co1
            if(form.cleaned_data['co1_content_type'] == 'document'):
                cr.page = form.cleaned_data['co1_page']
            cr.save()
            
            counter_more_cos = int(request.POST['counter_more_cos'])
            
            for num_id in range(2, counter_more_cos + 2):
                if 'co' + str(num_id) + '_id' in request.POST:
                    co_content_type = ContentType.objects.get(app_label="public_project", model=request.POST['co' + str(num_id) + '_content_type'])
                    co = co_content_type.get_object_for_this_type(id=request.POST['co' + str(num_id) + '_id'])
                    cr = CommentRelation()
                    cr.comment = c
                    cr.content_object = co
                    if(request.POST['co' + str(num_id) + '_content_type'] == 'document'):
                        cr.page = request.POST['co' + str(num_id) + '_page']
                    cr.save()
            
            email_users = User.objects.filter(userprofile__receive_new_comment_emails=True)
            
            #try:
            for i in range(0,1):
                for user in email_users:
                    sep = "-----------------------------------------------------------\n"
                    subject = _("NEW_COMMENT_EMAIL_SUBJECT") + ': ' + unicode(c)
                    
                    msg  = _("NEW_COMMENT_EMAIL_MESSAGE") + "\n"
                    msg += 'http://%s%s' % (Site.objects.get_current().domain, c.get_absolute_url()) + "\n\n"
                    msg += sep
                    msg += _("Name") + ": " + unicode(c.username) + "\n"
                    msg += _("E-Mail") + ": " + unicode(c.email) + "\n"
                    if c.feedback_allowed:
                        fa_str = _("yes")
                    else:
                        fa_str = _("no")
                    msg += _("Questions via mail allowed") + ": " + fa_str + "\n"
                    msg += _("Comment") + ":\n"
                    msg += c.comment + "\n" + sep
                    
                    msg += _("Comment on") + ":" + "\n"
                    for cr in c.commentrelation_set.all():
                        msg += unicode(cr.content_object) + "\n"
                        msg += 'http://%s%s' % (Site.objects.get_current().domain, cr.content_object.get_absolute_url()) + "\n"
                    
                    msg += "\n"
                    
                    if user.has_perm('public_project.change_comment') and user.email:
                        msg += _("NEW_COMMENT_EMAIL_MESSAGE_ACTIVATION") + "\n"
                        msg += 'http://%s/%s?activation_hash=%s&user=%s' \
                            % (Site.objects.get_current().domain, _("activate_comment_url"), c.activation_hash, urllib.quote_plus(unicode(user))) + "\n"
                    
                    send_mail(subject, msg, settings.EMAIL_FROM, [user.email], fail_silently=False)
            #except AttributeError:
            #    pass
            
            return "SENT"
        else:
            print form.errors
            return "FAILED"
    return ""


def merge_with_search_tag_docs(document_list, object):
    for tag in object.search_tags.all():
        search_docs = search_for_documents(tag.name)
        for doc in search_docs:
            if doc in document_list:
                index = document_list.index(doc)
                if not hasattr(document_list[index], 'search_tags'):
                    document_list[index].search_tags = []
                document_list[index].search_tags.append(tag.name)
            else:
                document_list.append(doc)
    document_list.sort(key=lambda x:x.title)
    return document_list


def index(request):
    research_request_list = ResearchRequest.objects.all()
    comment_list = Comment.objects.filter(published=True)
    
    if Event.objects.count() > 0:
        latest_event = Event.objects.all()[0]
    else:
        latest_event = None
    if Document.objects.count() > 0:
        latest_document = Document.objects.all()[0]
    else:
        latest_document = None 
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'site_category': SiteCategory.objects.get_or_create(category='home')[0],
        'current_project_goal_group': ProjectGoalGroup.objects.get_current(),
        'project_part_list': ProjectPart.objects.all(),
        'latest_event': latest_event,
        'latest_document': latest_document,
        'activity_list': ActivityLog.objects.all()[0:5],
        'research_request_list': research_request_list[0:3],
        'num_total_research_requests': len(research_request_list),
        'comment_list': comment_list[0:1],
        'num_total_comments': len(comment_list),
    })
    return render_to_response('index.html', context)


def project_parts(request):
    site_category = SiteCategory.objects.get_or_create(category='project_parts')[0]
    
    main_project_parts = ProjectPart.objects.annotate(count=Count('main_project_parts')).filter(count=0)
    middle = int(math.ceil(float(len(main_project_parts))/float(2)))
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'site_category': site_category,
        'main_project_part_list_left': main_project_parts[0:middle],
        'main_project_part_list_right': main_project_parts[middle:],
    })
    return render_to_response('project_parts.html', context)


def project_part(request, project_part_id):
    project_part = get_object_or_404(ProjectPart, pk=project_part_id)
    
    comment_form_status = validate_comment_form(request)
    
    content_type = ContentType.objects.get(app_label="public_project", model="projectpart")
    comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=project_part.id).filter(published=True).distinct()
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'site_category': SiteCategory.objects.get_or_create(category='project_parts')[0],
        'user_comment': get_user_comment(request),
        'project_part': project_part,
        'question_list': project_part.get_questions(),
        'event_list': project_part.get_events(),
        'document_list': project_part.get_documents(),
        'content_document_list': merge_with_search_tag_docs([], project_part),
        'comment_form_status': comment_form_status,
        'comment_list': comment_list[0:3],
        'num_total_comments': len(comment_list),
    })
    return render_to_response('project_part.html', context)


def goals(request):
    
    all_main_project_parts = ProjectPart.objects.annotate(count=Count('main_project_parts')).filter(count=0)
    main_project_parts = []
    for main_pp in all_main_project_parts:
        if main_pp.projectgoalgroup_set.count() > 0:
            main_project_parts.append(main_pp)
    middle = int(math.ceil(float(len(main_project_parts))/float(2)))
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'site_category': SiteCategory.objects.get_or_create(category='goals')[0],
        'common_goal_group_list': ProjectGoalGroup.objects.filter(project_part=None).order_by('event'),
        'main_project_part_list_left': main_project_parts[0:middle],
        'main_project_part_list_right': main_project_parts[middle:],
    })
    return render_to_response('goals.html', context)


def questions(request):
    all_mpps = ProjectPart.objects.annotate(count=Count('main_project_parts')).filter(count=0)
    main_project_parts = []
    for mpp in all_mpps:
        if mpp.get_num_questions() > 0:
            main_project_parts.append(mpp)
    middle = int(math.ceil(float(len(main_project_parts))/float(2)))
    
    research_request_list = ResearchRequest.objects.all()
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'site_category': SiteCategory.objects.get_or_create(category='questions')[0],
        'main_project_part_list_left': main_project_parts[0:middle],
        'main_project_part_list_right': main_project_parts[middle:],
        'research_request_list': research_request_list[0:3],
        'num_total_research_requests': len(research_request_list),
    })
    return render_to_response('questions.html', context)


def question(request, question_id):
    question = get_object_or_404(Question, pk=question_id)
    
    research_request_form_status = validate_research_request_form(request)
    content_type = ContentType.objects.get(app_label="public_project", model="question")
    research_request_list = ResearchRequest.objects.filter(researchrequestrelation__content_type=content_type).filter(researchrequestrelation__object_id=question.id).distinct()
    
    comment_form_status = validate_comment_form(request)
    content_type = ContentType.objects.get(app_label="public_project", model="question")
    comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=question.id).filter(published=True).distinct()
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'site_category': SiteCategory.objects.get_or_create(category='questions')[0],
        'user_comment': get_user_comment(request),
        'research_request': get_research_request(request),
        'question': question,
        'document_list': question.documents.all(),
        'research_request_form_status': research_request_form_status,
        'research_request_list': research_request_list[0:3],
        'num_total_research_requests': len(research_request_list),
        'comment_form_status': comment_form_status,
        'comment_list': comment_list[0:3],
        'num_total_comments': len(comment_list),
    })
    return render_to_response('question.html', context)


def participants(request):
    site_category = SiteCategory.objects.get_or_create(category='participants')[0]
    participant_types = ParticipantType.objects.all()
    middle = int(math.ceil(float(len(participant_types))/float(2)))
    #groups = Participant.objects.annotate(count=Count('belongs_to')).filter(count=0)
    #middle = int(math.ceil(float(len(groups))/float(2)))
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'site_category': site_category,
        'participant_type_list_left': participant_types[0:middle],
        'participant_type_list_right': participant_types[middle:],
        #'group_list_left': groups[0:middle],
        #'group_list_right': groups[middle:],
    })
    return render_to_response('participants.html', context)


def participant(request, participant_id):
    participant = get_object_or_404(Participant, pk=participant_id)
    
    comment_form_status = validate_comment_form(request)
    content_type = ContentType.objects.get(app_label="public_project", model="participant")
    comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=participant.id).filter(published=True).distinct()
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'site_category': SiteCategory.objects.get_or_create(category='participants')[0],
        'user_comment': get_user_comment(request),
        'participant': participant,
        'question_list': participant.get_questions(),
        'event_list': participant.get_events(),
        'document_list': participant.get_documents(),
        'content_document_list': merge_with_search_tag_docs([], participant),
        'comment_form_status': comment_form_status,
        'comment_list': comment_list[0:3],
        'num_total_comments': len(comment_list),
    })
    return render_to_response('participant.html', context)


def events(request):
    site_category = SiteCategory.objects.get_or_create(category='events')[0]
    
    all_mpps = ProjectPart.objects.annotate(count=Count('main_project_parts')).filter(count=0)
    main_project_parts = []
    for mpp in all_mpps:
        if mpp.get_num_events() > 0:
            main_project_parts.append(mpp)
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'site_category': SiteCategory.objects.get_or_create(category='events')[0],
        'project_goal_group_list': ProjectGoalGroup.objects.all().order_by('event'),
        'chronology_list': Event.objects.all(),
        'main_project_part_list': main_project_parts,
    })
    return render_to_response('events.html', context)


def event(request, event_id):
    event = get_object_or_404(Event, pk=event_id)
    
    content_document_list = merge_with_search_tag_docs([], event)
    
    comment_form_status = validate_comment_form(request)
    content_type = ContentType.objects.get(app_label="public_project", model="event")
    comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=event.id).filter(published=True).distinct()
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'site_category': SiteCategory.objects.get_or_create(category='events')[0],
        'user_comment': get_user_comment(request),
        'event': event,
        'document_list': event.related_documents.order_by("title"),
        'content_document_list': content_document_list,
        'comment_form_status': comment_form_status,
        'comment_list': comment_list[0:3],
        'num_total_comments': len(comment_list),
    })
    return render_to_response('event.html', context)


def web_source(request, web_source_id):
    web_source = get_object_or_404(WebSource, pk=web_source_id)
    
    req = urllib2.Request(web_source.url, headers={'User-Agent' : "Magic Browser"}) 
    response = urllib2.urlopen(req)
    html = response.read()
    
    search_tags = SearchTag.objects.all()
    found_search_tag_list = []
    for st in search_tags:
        if st.name.lower() in unicode(html, errors='replace').lower():
            found_search_tag_list.append(st)
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'web_source': web_source,
        'found_search_tag_list': found_search_tag_list,
    })
    if request.user.is_authenticated():
        return render_to_response('web_source.html', context)
    else:
        raise Http404


def documents(request):
    site_category = SiteCategory.objects.get_or_create(category='documents')[0]
    
    all_mpps = ProjectPart.objects.annotate(count=Count('main_project_parts')).filter(count=0)
    main_project_parts = []
    for mpp in all_mpps:
        if mpp.get_num_documents() > 0:
            main_project_parts.append(mpp)
    middle = int(math.ceil(float(len(main_project_parts))/float(2)))
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'site_category': site_category,
        'main_project_part_list_left': main_project_parts[0:middle],
        'main_project_part_list_right': main_project_parts[middle:],
    })
    return render_to_response('documents.html', context)


def xhr_universal_search(request):
    if request.method == 'GET' and 'query' in request.GET:
        query_string = request.GET['query']
        query_string = unicode(query_string).encode('utf-8')
        
        if len(query_string) >= 4:
            entry_query = get_query(query_string, ['name',])
            p_list = list(Participant.objects.select_related().filter(entry_query)[0:10])
            
            entry_query = get_query(query_string, ['name',])
            pp_list = list(ProjectPart.objects.select_related().filter(entry_query)[0:10])
            
            entry_query = get_query(query_string, ['title',])
            q_list = list(Question.objects.select_related().filter(entry_query)[0:10])
            
            entry_query = get_query(query_string, ['title',])
            e_list = list(Event.objects.select_related().filter(entry_query)[0:10])
            
            entry_query = get_query(query_string, ['title',])
            d_list = list(Document.objects.select_related().filter(entry_query)[0:10])
            
            object_list = p_list + pp_list + q_list + e_list + d_list
        else:
            object_list = []
        
        res = {
            'values': {},
            'options': [],
        }
        print query_string
        if 'with_query_search' in request.GET:
            if len(query_string) < 4:
                option = query_string
            else:
                option = '<i class="icon-search"></i> ' + query_string
            res['values'][option] = {
                'content_type': 'query_search',
                'id': 0,
                'absolute_url': '/' + _('search_url') + '?q=' + urllib.quote_plus(query_string),
            }
            res['options'].append(option)
        
        for object in object_list:
            content_type = object.__class__.__name__.lower()
            id = object.id
            
            if not 'ommit_content_type' in request.GET or not (content_type == request.GET['ommit_content_type'] and str(id) == request.GET['ommit_id']):
                option = '<i class="' + object.get_icon_class() + '"></i> ' + unicode(object)
                res['values'][option] = {
                    'content_type': content_type,
                    'id': id,
                    'absolute_url': object.get_absolute_url(),
                }
                res['options'].append(option)
        
        mimetype = 'application/javascript'
        return HttpResponse(json.dumps(res), mimetype)


def xhr_document_tags(request):
    if request.method == 'POST':
        
        colors = {
            'projectpart': '#0d9434',
            'participant': '#3e3ec7',
            'event': '#c91a1a',
        }
        if request.POST['document_id']:
            document = get_object_or_404(Document, pk=request.POST['document_id'])
            if request.POST['content_type']:
                st_dict_list = SearchTagCacheEntry.objects.filter(document=document).filter(tag__content_type__model=request.POST['content_type']).values('tag__name', 'tag__content_type__model',).annotate(num_tags=Count('tag__name')).order_by('-num_tags')
                #cache_entries = SearchTagCacheEntry.objects.filter(document=document).filter(tag__content_type__model=request.POST['content_type'])
            else:
                st_dict_list = SearchTagCacheEntry.objects.filter(document=document).values('tag__name', 'tag__content_type__model',).annotate(num_tags=Count('tag__name')).order_by('-num_tags')
            st_dict_list = st_dict_list[0:16]
            size_start = 10
            size_span = 10
        else:
            if request.POST['content_type']:
                st_dict_list = SearchTagCacheEntry.objects.filter(tag__content_type__model=request.POST['content_type']).values('tag__name', 'tag__content_type__model',).annotate(num_tags=Count('tag__name')).order_by('-num_tags')[0:22]
            else:
                st_dict_list = SearchTagCacheEntry.objects.values('tag__name', 'tag__content_type__model',).annotate(num_tags=Count('tag__name')).order_by('-num_tags')[0:22]
            size_start = 12
            size_span = 12
        
        if len(st_dict_list) > 0:
            first = st_dict_list[0]
            max_num_results = first['num_tags']
        
        words = []
        for st_dict in st_dict_list:
            text = st_dict['tag__name']
            size = size_start + int(round((float(st_dict['num_tags'])/ float(max_num_results)) * size_span))
            color = colors[st_dict['tag__content_type__model']]
            
            word = { "text": text, "size":size, "color": color, }
            words.append(word)
        
        data = json.dumps(words)
        
        mimetype = 'application/javascript'
        return HttpResponse(data, mimetype)


def document(request, document_id):
    document = get_object_or_404(Document, pk=document_id)
    comment_form_status = validate_comment_form(request)
    
    query_string = ''
    found_pages = []
    if ('q' in request.GET) and request.GET['q'].strip():
        query_string = request.GET['q']
        entry_query = get_query(query_string, ['document__title', 'content',])
        found_pages = Page.objects.select_related().filter(document=document).filter(entry_query).order_by('number')
    
    content_type = ContentType.objects.get(app_label="public_project", model="document")
    tmp_comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=document.id).filter(published=True).distinct()
    print len(tmp_comment_list)
    comment_list = []
    for comment in tmp_comment_list:
        print type(comment)
        cr = comment.commentrelation_set.filter(document=document)[0]
        comment.page = cr.page
        comment_list.append(comment)
    comment_list.sort(key=lambda x:x.page)
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'site_category': SiteCategory.objects.get_or_create(category='documents')[0],
        'user_comment': get_user_comment(request),
        'document': document,
        'found_pages': found_pages,
        'comment_form_status': comment_form_status,
        'comment_list': comment_list,
        'num_total_comments': len(comment_list),
        'query_string': query_string,
    })
    return render_to_response('document.html', context)


def search(request):
    
    if ('q' in request.GET) and request.GET['q'].strip():
        
        query_string = request.GET['q']
        
        entry_query = get_query(query_string, ['name', 'description',])
        project_part_list = ProjectPart.objects.select_related().filter(entry_query)
        
        entry_query = get_query(query_string, ['title', 'description',])
        question_list = Question.objects.select_related().filter(entry_query)
        
        entry_query = get_query(query_string, ['name', 'description',])
        participant_list = Participant.objects.select_related().filter(entry_query)
        
        entry_query = get_query(query_string, ['title', 'description',])
        event_list = Event.objects.select_related().filter(entry_query)
        
        document_list = search_for_documents(query_string)
        
        context = RequestContext(request, {
            'site_config': SiteConfig.objects.get_site_config(request),
            'query': query_string,
            'project_part_list': project_part_list,
            'question_list': question_list,
            'participant_list': participant_list,
            'event_list': event_list,
            'document_list': document_list,
            'q': query_string,
        })
        
        return render_to_response('search.html', context)
    else:
        return HttpResponse("An Error occured!")


def research_requests(request, object_id, content_type):
    
    if not content_type:
        object = None
        research_request_list = ResearchRequest.objects.all()
    
    if content_type == 'question':
        object = get_object_or_404(Question, pk=object_id)
        content_type = ContentType.objects.get(app_label="public_project", model="question")
        research_request_list = ResearchRequest.objects.filter(researchrequestrelation__content_type=content_type).filter(researchrequestrelation__object_id=object.id).distinct()
    
    ph_title = unicode(object)
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'rr_object': object,
        'research_request_list': research_request_list,
        'num_total_research_requests': len(research_request_list),
        'content_type': content_type,
        'ph_title': ph_title,    
    })    
    
    return render_to_response('research_requests.html', context)


def comments(request, object_id, content_type):
    
    if not content_type:
        object = None
        comment_list = Comment.objects.filter(published=True)
    
    if content_type == 'project_part':
        object = get_object_or_404(ProjectPart, pk=object_id)
        content_type = ContentType.objects.get(app_label="public_project", model="projectpart")
        comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=object.id).filter(published=True).distinct()
    if content_type == 'question':
        object = get_object_or_404(Question, pk=object_id)
        content_type = ContentType.objects.get(app_label="public_project", model="question")
        comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=object.id).filter(published=True).distinct()
    if content_type == 'participant':
        object = get_object_or_404(Participant, pk=object_id)
        content_type = ContentType.objects.get(app_label="public_project", model="participant")
        comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=object.id).filter(published=True).distinct()
    if content_type == 'event':
        object = get_object_or_404(Event, pk=object_id)
        content_type = ContentType.objects.get(app_label="public_project", model="event")
        comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=object.id).filter(published=True).distinct()
    if content_type == 'document':
        object = get_object_or_404(Document, pk=object_id)
        content_type = ContentType.objects.get(app_label="public_project", model="document")
        comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=object.id).filter(published=True).distinct()
    
    ph_title = unicode(object)
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'commented_object': object,
        'comment_list': comment_list,
        'num_total_comments': len(comment_list),
        'content_type': content_type,
        'ph_title': ph_title,    
    })    
    
    return render_to_response('comments.html', context)


def api(request):
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
    })
    return render_to_response('api.html', context)


def contact(request):
    image_list = Image.objects.all()
    
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
        'image_list': image_list,
    })
    return render_to_response('contact.html', context)


def activate_comment(request):
    c = get_object_or_404(Comment, activation_hash=request.GET['activation_hash'])
    site_config = SiteConfig.objects.get_site_config(request)
    
    res  = '<!DOCTYPE html><html><head><meta charset="utf-8"></head><body>'
    res += '<div style="margin:20px;padding:20px;border:1px solid #999;float:left;color:#333;font-size:14px;'
    res += 'font-family:arial, helvetica, sans-serif;">'
    
    if not c.published:
        c.published = True
        c.published_by = request.GET['user']
        c.save()
        res += _("The following comment was activated for publication on website:") + '<br><br>'
        res += unicode(c)
        
        #User mail
        subject = _("Your comment on %s was published") % site_config.short_title
                    
        msg  = _("Hello %s,") % c.username + "\n\n"
        msg += _("thank you for your comment, which you can find under the following url:") + "\n"
        msg += 'http://%s%s' % (Site.objects.get_current().domain, c.get_absolute_url()) + "\n\n"
        
        msg += _("You can use the url above to tell others about your comment.") + "\n\n"
        
        msg += _("If you want to share your comment on a social network,") + "\n"
        msg += _("you can use the following share urls as a staring point:") + "\n\n"
        
        msg += "Twitter:\n" + c.get_twitter_url() + "\n\n"
        msg += "Facebook:\n" + c.get_facebook_url() + "\n\n"
        msg += "Google+:\n" + c.get_google_plus_url() + "\n\n"
        msg += "App.net:\n" + c.get_app_net_url() + "\n\n\n"
        
        msg += _("Greetings") + "\n\n"
        msg += _("Your %s team") % site_config.short_title + "\n"

        send_mail(subject, msg, settings.EMAIL_FROM, [c.email,], fail_silently=False)
        
        al = ActivityLog()
        al.content_object = c
        al.type = 'NC'
        al.save()
        
    else:
        res += _("Comment already activated by user %s.") % (c.published_by)
    
    res += '</div></body></html>'
    
    return HttpResponse(res)


def custom_404_view(request):
    context = RequestContext(request, {
        'site_config': SiteConfig.objects.get_site_config(request),
    })
    return render_to_response('404.html', context)
    
    


from django.contrib.contenttypes.models import ContentType
from django.contrib.syndication.views import Feed, FeedDoesNotExist
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext as _
from public_project.models import SiteConfig, ProjectPart, Question, Participant, Event, Document, ResearchRequest, Comment, ActivityLog


class ActivityFeed(Feed):
    item_guid_is_permalink = False
    
    def __init__(self, type, link):
        self.type = type
        self.link = '/' + link
        if SiteConfig.objects.count() == 0:
            return None
        site_config = SiteConfig.objects.get_site_config(None)
        if type == 'NA':
            self.title = site_config.short_title + ": " + _("Subject Areas")
            self.description = _("Feed with new subject areas from %s.") % site_config.short_title
        if type == 'NQ':
            self.title = site_config.short_title + ": " + _("Questions")
            self.description = _("Feed with new questions from %s.") % site_config.short_title
        if type == 'PA':
            self.title = site_config.short_title + ": " + _("Participants")
            self.description = _("Feed with new participants from %s.") % site_config.short_title
        if type == 'NE':
            self.title = site_config.short_title + ": " + _("Events")
            self.description = _("Feed with new events from %s.") % site_config.short_title
        if type == 'ND':
            self.title = site_config.short_title + ": " + _("Documents")
            self.description = _("Feed with new documents from %s.") % site_config.short_title
        if type  == None:
            self.title = site_config.short_title + ": " + _("Activities")
            self.description = _("General feed with all activities from %s.") % site_config.short_title
        super(ActivityFeed, self).__init__()
    
    def items(self):
        if self.type == None:
            return ActivityLog.objects.all()[:8]
        else:
            return ActivityLog.objects.filter(type=self.type)[:8]
    
    def item_title(self, item):
        title  = ''
        if self.type == None:
            title += item.get_type_display() + ": "
        title += unicode(item.content_object) 
        return title
    
    def item_description(self, item):
        if item.content_object:
            return item.content_object.get_feed_description()
        else:
            return ''
    
    def item_link(self, item):
        if item.content_object:
            return item.content_object.get_absolute_url()
        else:
            return ''
    
    def item_guid(self, item):
        if item.content_object:
            return item.content_object.get_absolute_url()
        else:
            return ''
    
    def item_pubdate(self, item):
        return item.date


class QuestionResearchRequestsFeed(Feed):
    item_guid_is_permalink = False
    
    def get_object(self, request, object_id):
        obj = get_object_or_404(Question, pk=object_id)
        self.set_config(obj)
        return obj
    
    def items(self, obj):
        content_type = ContentType.objects.get(app_label="public_project", model="question")
        research_request_list = ResearchRequest.objects.filter(researchrequestrelation__content_type=content_type).filter(researchrequestrelation__object_id=obj.id).distinct()
        return research_request_list[:8]
    
    def set_config(self, obj):
        site_config = SiteConfig.objects.get_site_config(None)
        self.link = obj.get_absolute_url() + _('research_requests_url')
        self.title = site_config.short_title + ": " + _("Research Requests on") + " " + unicode(obj)
        self.description = _("Feed with new research requests on {object} on {title}.").format(
            object = unicode(obj),
            title = site_config.short_title,
        )
    
    def item_title(self, item):
        return unicode(item)
    
    def item_description(self, item):
        return item.get_feed_description()
    
    def item_link(self, item):
        return item.get_absolute_url()
    
    def item_guid(self, item):
        return item.get_absolute_url()
    
    def item_pubdate(self, item):
        return item.date_added


class ObjectCommentsFeed(Feed):
    item_guid_is_permalink = False
    
    def set_config(self, obj):
        site_config = SiteConfig.objects.get_site_config(None)
        self.link = obj.get_absolute_url() + _('comments_url')
        self.title = site_config.short_title + ": " + _("Comments on") + " " + unicode(obj)
        self.description = _("Feed with new comments on {object} on {title}.").format(
            object = unicode(obj),
            title = site_config.short_title,
        )
    
    def item_title(self, item):
        return unicode(item)
    
    def item_description(self, item):
        return item.get_feed_description()
    
    def item_link(self, item):
        return item.get_absolute_url()
    
    def item_guid(self, item):
        return item.get_absolute_url()
    
    def item_pubdate(self, item):
        return item.date_added


class ProjectPartCommentsFeed(ObjectCommentsFeed):
    
    def get_object(self, request, object_id):
        obj = get_object_or_404(ProjectPart, pk=object_id)
        self.set_config(obj)
        return obj
    
    def items(self, obj):
        content_type = ContentType.objects.get(app_label="public_project", model="projectpart")
        comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=obj.id).filter(published=True).distinct()
        return comment_list[:8]


class QuestionCommentsFeed(ObjectCommentsFeed):
    
    def get_object(self, request, object_id):
        obj = get_object_or_404(Question, pk=object_id)
        self.set_config(obj)
        return obj
    
    def items(self, obj):
        content_type = ContentType.objects.get(app_label="public_project", model="question")
        comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=obj.id).filter(published=True).distinct()
        return comment_list[:8]


class ParticipantCommentsFeed(ObjectCommentsFeed):
    
    def get_object(self, request, object_id):
        obj = get_object_or_404(Participant, pk=object_id)
        self.set_config(obj)
        return obj
    
    def items(self, obj):
        content_type = ContentType.objects.get(app_label="public_project", model="participant")
        comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=obj.id).filter(published=True).distinct()
        return comment_list[:8]


class EventCommentsFeed(ObjectCommentsFeed):
    
    def get_object(self, request, object_id):
        obj = get_object_or_404(Event, pk=object_id)
        self.set_config(obj)
        return obj
    
    def items(self, obj):
        content_type = ContentType.objects.get(app_label="public_project", model="event")
        comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=obj.id).filter(published=True).distinct()
        return comment_list[:8]
    

class DocumentCommentsFeed(ObjectCommentsFeed):
    
    def get_object(self, request, object_id):
        obj = get_object_or_404(Document, pk=object_id)
        self.set_config(obj)
        return obj
    
    def items(self, obj):
        content_type = ContentType.objects.get(app_label="public_project", model="document")
        comment_list = Comment.objects.filter(commentrelation__content_type=content_type).filter(commentrelation__object_id=obj.id).filter(published=True).distinct()
        return comment_list[:8]


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        db.rename_column(u'public_project_image', 'attribution_html', 'attribution')


    def backwards(self, orm):
        db.rename_column(u'public_project_image', 'attribution', 'attribution_html')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'public_project.activitylog': {
            'Meta': {'ordering': "['-date']", 'object_name': 'ActivityLog'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'info': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '2'})
        },
        u'public_project.comment': {
            'Meta': {'ordering': "['-date_added']", 'object_name': 'Comment'},
            'activation_hash': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'comment': ('django.db.models.fields.TextField', [], {}),
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '250'}),
            'feedback_allowed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'published_by': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '250'})
        },
        u'public_project.commentrelation': {
            'Meta': {'object_name': 'CommentRelation'},
            'comment': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['public_project.Comment']"}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'page': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        u'public_project.document': {
            'Meta': {'ordering': "['-date_added']", 'object_name': 'Document'},
            'comments': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'date': ('django.db.models.fields.DateField', [], {}),
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {}),
            'document': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'events': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'related_documents'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['public_project.Event']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'participants': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'related_documents'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['public_project.Participant']"}),
            'pdf_images_generated': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'project_parts': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'related_documents'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['public_project.ProjectPart']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250'})
        },
        u'public_project.event': {
            'Meta': {'ordering': "['-date']", 'object_name': 'Event'},
            'comments': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'date': ('django.db.models.fields.DateField', [], {}),
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {}),
            'event_type': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'important': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'participants': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'related_events'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['public_project.Participant']"}),
            'project_parts': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'related_events'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['public_project.ProjectPart']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250'})
        },
        u'public_project.image': {
            'Meta': {'ordering': "['title']", 'object_name': 'Image'},
            'attribution': ('django.db.models.fields.CharField', [], {'max_length': '250'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250'})
        },
        u'public_project.membership': {
            'Meta': {'object_name': 'Membership'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'from_participant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'from_memberships'", 'to': u"orm['public_project.Participant']"}),
            'function': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'to_participant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'to_memberships'", 'to': u"orm['public_project.Participant']"})
        },
        u'public_project.page': {
            'Meta': {'ordering': "['number']", 'object_name': 'Page'},
            'content': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'document': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['public_project.Document']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'number': ('django.db.models.fields.IntegerField', [], {})
        },
        u'public_project.participant': {
            'Meta': {'ordering': "['order', 'name']", 'object_name': 'Participant'},
            'belongs_to': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['public_project.Participant']", 'through': u"orm['public_project.Membership']", 'symmetrical': 'False'}),
            'comments': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '250'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '500', 'null': 'True', 'blank': 'True'})
        },
        u'public_project.projectgoal': {
            'Meta': {'ordering': "['order']", 'object_name': 'ProjectGoal'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '100', 'null': 'True', 'blank': 'True'}),
            'performance_figure': ('django.db.models.fields.CharField', [], {'max_length': '250'}),
            'project_goal_group': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['public_project.ProjectGoalGroup']"})
        },
        u'public_project.projectgoalgroup': {
            'Meta': {'object_name': 'ProjectGoalGroup'},
            'comments': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {}),
            'event': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['public_project.Event']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_current': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'project_part': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['public_project.ProjectPart']", 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250'})
        },
        u'public_project.projectpart': {
            'Meta': {'ordering': "['order', 'name']", 'object_name': 'ProjectPart'},
            'comments': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'main_project_parts': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': u"orm['public_project.ProjectPart']", 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '500', 'null': 'True', 'blank': 'True'})
        },
        u'public_project.question': {
            'Meta': {'ordering': "['title']", 'object_name': 'Question'},
            'answer': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'answered': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'comments': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {}),
            'documents': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'related_documents'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['public_project.Document']"}),
            'events': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'related_questions'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['public_project.Event']"}),
            'explanations': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'participants': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'related_questions'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['public_project.Participant']"}),
            'project_parts': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'related_questions'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['public_project.ProjectPart']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250'})
        },
        u'public_project.researchrequest': {
            'Meta': {'ordering': "['-date_added']", 'object_name': 'ResearchRequest'},
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'nr': ('django.db.models.fields.CharField', [], {'max_length': '8'}),
            'open': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250'})
        },
        u'public_project.researchrequestrelation': {
            'Meta': {'object_name': 'ResearchRequestRelation'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'page': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'research_request': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['public_project.ResearchRequest']"})
        },
        u'public_project.searchtag': {
            'Meta': {'ordering': "['order']", 'object_name': 'SearchTag'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '100', 'null': 'True', 'blank': 'True'})
        },
        u'public_project.searchtagcacheentry': {
            'Meta': {'ordering': "['-num_results']", 'object_name': 'SearchTagCacheEntry'},
            'document': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['public_project.Document']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'num_results': ('django.db.models.fields.IntegerField', [], {}),
            'tag': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['public_project.SearchTag']"})
        },
        u'public_project.sitecategory': {
            'Meta': {'object_name': 'SiteCategory'},
            'category': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '50'}),
            'comments': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'documents': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'related_site_categories'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['public_project.Document']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'intro_text': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'})
        },
        u'public_project.siteconfig': {
            'Meta': {'object_name': 'SiteConfig'},
            'comments': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'contact_html': ('django.db.models.fields.TextField', [], {'default': "u'This text will be shown on the contact page.'"}),
            'desc_about': ('django.db.models.fields.TextField', [], {'default': "u'About text'"}),
            'footer_html': ('django.db.models.fields.TextField', [], {'default': "u'This text will be shown in the footer of the site.'"}),
            'header_image': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['public_project.Image']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'intro_text': ('django.db.models.fields.TextField', [], {'default': "u'This is a project watch website.'"}),
            'navi_link_color': ('django.db.models.fields.CharField', [], {'default': "'#FFFFFF'", 'max_length': '7'}),
            'short_title': ('django.db.models.fields.CharField', [], {'default': "u'ProjectWatch'", 'max_length': '250'}),
            'sub_title': ('django.db.models.fields.CharField', [], {'default': "u'Project Website Subtitle'", 'max_length': '250'}),
            'sub_title_color': ('django.db.models.fields.CharField', [], {'default': "'#444444'", 'max_length': '7'}),
            'title': ('django.db.models.fields.CharField', [], {'default': "u'ProjectWatch'", 'max_length': '250'}),
            'title_color': ('django.db.models.fields.CharField', [], {'default': "'#990000'", 'max_length': '7'})
        },
        u'public_project.userprofile': {
            'Meta': {'object_name': 'UserProfile'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'receive_new_comment_emails': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'user': ('django.db.models.fields.related.OneToOneField', [], {'to': u"orm['auth.User']", 'unique': 'True'})
        },
        u'public_project.websource': {
            'Meta': {'ordering': "['order']", 'object_name': 'WebSource'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'date_added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '100', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        }
    }

    complete_apps = ['public_project']



from django import template

register = template.Library()

@register.filter
def modelname(object):
    if hasattr(object, '__class__'):
        return object.__class__.__name__.lower()
    else:
        return ""
    


from django import forms
from django.utils.translation import ugettext_lazy as _
    

class CommentForm(forms.Form):
    username = forms.CharField()
    email = forms.EmailField()
    comment = forms.CharField(max_length=800)
    feedback_allowed = forms.BooleanField(required=False)
    co1_id = forms.IntegerField()
    co1_content_type = forms.CharField()
    co1_page = forms.IntegerField(required=False)
    


from django.conf import settings
from django.conf.urls import include, patterns, url
from django.utils.translation import ugettext as _
from tastypie.api import Api
from public_project.api import *
from public_project.feeds import *

handler404 = 'public_project.views.custom_404_view'

urlpatterns = patterns('',
)

if getattr(settings, 'DPP_PUBLIC_API', False):
    v01_api = Api(api_name='v0.1')
    v01_api.register(ProjectPartsResource())
    v01_api.register(QuestionsResource())
    v01_api.register(ParticipantsResource())
    v01_api.register(EventsResource())
    v01_api.register(PagesResource())
    v01_api.register(DocumentsResource())
    
    urlpatterns = patterns('',
        url(r'^api/$', 'public_project.views.api'),
        (r'^api/', include(v01_api.urls)),
    )

urlpatterns += patterns('',
    url(r'^xhr/universal_search/', 'public_project.views.xhr_universal_search', name='xhr_universal_search'),
    url(r'^xhr/document_tags/', 'public_project.views.xhr_document_tags', name='xhr_document_tags'),
    url(r'^$', 'public_project.views.index'),
    url(r'^rss/$', ActivityFeed(None, '')),
    url(r'^%s$' % _('comments_url'), 'public_project.views.comments', {'object_id': None, 'content_type': None}),
    url(r'^%srss/$' % _('comments_url'), ActivityFeed('NC', _('comments_url'))),
    url(r'^404test/$', 'public_project.views.custom_404_view'),
    url(r'^%s$' % _('project_parts_url'), 'public_project.views.project_parts'),
    url(r'^%srss/$' % _('project_parts_url'), ActivityFeed('NA', _('project_parts_url'))),
    url(r'^%s(?P<project_part_id>\d+)/$' % _('project_parts_url'), 'public_project.views.project_part'),
    url(r'^%s(?P<object_id>\d+)/%s$' % (_('project_parts_url'), _('comments_url')), 'public_project.views.comments', {'content_type': 'project_part'}),
    url(r'^%s(?P<object_id>\d+)/%srss/$' % (_('project_parts_url'), _('comments_url')), ProjectPartCommentsFeed()),
    url(r'^%s$' % _('goals_url'), 'public_project.views.goals'),
    url(r'^%s$' % _('questions_url'), 'public_project.views.questions'),
    url(r'^%s%s$' % (_('questions_url'), _('research_requests_url')), 'public_project.views.research_requests', {'object_id': None, 'content_type': None}),
    url(r'^%s%srss/$' % (_('questions_url'), _('research_requests_url')), ActivityFeed('RR', _('research_requests_url'))),
    url(r'^%s(?P<question_id>\d+)/$' % _('questions_url'), 'public_project.views.question'),
    url(r'^%s(?P<object_id>\d+)/%s$' % (_('questions_url'), _('research_requests_url')), 'public_project.views.research_requests', {'content_type': 'question'}),
    url(r'^%s(?P<object_id>\d+)/%srss/$' % (_('questions_url'), _('research_requests_url')), QuestionResearchRequestsFeed()),
    url(r'^%srss/$' % _('questions_url'), ActivityFeed('NQ', _('questions_url'))),
    url(r'^%s(?P<object_id>\d+)/%s$' % (_('questions_url'), _('comments_url')), 'public_project.views.comments', {'content_type': 'question'}),
    url(r'^%s(?P<object_id>\d+)/%srss/$' % (_('questions_url'), _('comments_url')), QuestionCommentsFeed()),
    url(r'^%s$' % _('events_url'), 'public_project.views.events'),
    url(r'^%srss/$' % _('events_url'), ActivityFeed('NE', _('events_url'))),
    url(r'^%s(?P<event_id>\d+)/$' % _('events_url'), 'public_project.views.event'),
    url(r'^%s(?P<object_id>\d+)/%s$' % (_('events_url'), _('comments_url')), 'public_project.views.comments', {'content_type': 'event'}),
    url(r'^%s(?P<object_id>\d+)/%srss/$' % (_('events_url'), _('comments_url')), EventCommentsFeed()),
    url(r'^%s$' % _('participants_url'), 'public_project.views.participants'),
    url(r'^%srss/$' % _('participants_url'), ActivityFeed('PA', _('participants_url'))),
    url(r'^%s(?P<participant_id>\d+)/$' % _('participants_url'), 'public_project.views.participant'),
    url(r'^%s(?P<object_id>\d+)/%s$' % (_('participants_url'), _('comments_url')), 'public_project.views.comments', {'content_type': 'participant'}),
    url(r'^%s(?P<object_id>\d+)/%srss/$' % (_('participants_url'), _('comments_url')), ParticipantCommentsFeed()),
    url(r'^%s(?P<web_source_id>\d+)/$' % _('web_sources_url'), 'public_project.views.web_source'),
    url(r'^%s$' % _('documents_url'), 'public_project.views.documents'),
    url(r'^%srss/$' % _('documents_url'), ActivityFeed('ND', _('documents_url'))),
    url(r'^%s(?P<document_id>\d+)/$' % _('documents_url'), 'public_project.views.document'),
    url(r'^%s(?P<object_id>\d+)/%s$' % (_('documents_url'), _('comments_url')), 'public_project.views.comments', {'content_type': 'document'}),
    url(r'^%s(?P<object_id>\d+)/%srss/$' % (_('documents_url'), _('comments_url')), DocumentCommentsFeed()),
    url(r'^%s$' %  _('search_url'), 'public_project.views.search'),
    url(r'^%s$' % _('contact_url'), 'public_project.views.contact'),
    url(r'^%s$' % _('activate_comment_url'), 'public_project.views.activate_comment'),
)
                       



from django import forms
from django.db.models import Count
from django.contrib import admin, messages
from django.contrib.admin import SimpleListFilter
from django.contrib.admin.actions import delete_selected
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import User
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext as _
from public_project.models import *


def get_num_search_tags(obj):
    num = obj.search_tags.count()
    if num > 0:
        return str(num)
    else:
        return ""

def get_num_web_sources(obj):
    num = obj.web_sources.count()
    if num > 0:
        return str(num)
    else:
        return ""

def get_num_project_parts(obj):
    num = obj.project_parts.count()
    if num > 0:
        return str(num)
    else:
        return ""

def get_num_participants(obj):
    num = obj.participants.count()
    if num > 0:
        return str(num)
    else:
        return ""

def get_num_events(obj):
    num = obj.events.count()
    if num > 0:
        return str(num)
    else:
        return ""

def get_num_documents(obj):
    num = obj.documents.count()
    if num > 0:
        return str(num)
    else:
        return ""

class UserProfileInline(admin.StackedInline):
    model = UserProfile
    can_delete = False
    verbose_name_plural = 'profile'


class UserAdmin(UserAdmin):
    inlines = (UserProfileInline, )


class ImageAdmin(admin.ModelAdmin):
    list_display = ('title', 'image_url', 'attribution_with_url',)
    search_fields = ['title',]
    
    def image_url(self, obj):
        return '<a href="' + obj.image.url + '" target="_blank">' + obj.image.url + '</a>'
    image_url.allow_tags = True
    image_url.short_description = _('Image')
    
    def attribution_with_url(self, obj):
        if obj.attribution_url:
            return '<a href="' + obj.attribution_url + '" target="_blank">' + obj.attribution + '</a>'
        else:
            return obj.attribution
    attribution_with_url.allow_tags = True
    attribution_with_url.short_description = _('Attribution')


class WebSourceInline(generic.GenericTabularInline):
    model = WebSource


class CustomSiteConfigAdminForm(forms.ModelForm):
    intro_text = forms.CharField(widget=forms.Textarea(attrs={'class':'htmleditor'}))
    about_text = forms.CharField(widget=forms.Textarea(attrs={'class':'htmleditor'}))
    footer = forms.CharField(widget=forms.Textarea(attrs={'class':'htmleditor'}))
    contact_text = forms.CharField(widget=forms.Textarea(attrs={'class':'htmleditor'}))


class SiteConfigAdmin(admin.ModelAdmin):
    list_display = ('title', 'short_title', 'title_color',)
    save_on_top = True
    form = CustomSiteConfigAdminForm


class CustomSiteCategoryAdminForm(forms.ModelForm):
    intro_text = forms.CharField(widget=forms.Textarea(attrs={'class':'htmleditor'}))


class SiteCategoryAdmin(admin.ModelAdmin):
    list_display = ('category', 'num_documents', 'num_web_sources')
    save_on_top = True
    inlines = [
        WebSourceInline,
    ]
    filter_horizontal = ('documents',)
    form = CustomSiteCategoryAdminForm
    
    def num_documents(self, obj):
        return get_num_documents(obj)
    num_documents.short_description = _('Documents')
    
    def num_web_sources(self, obj):
        return get_num_web_sources(obj)
    num_web_sources.short_description = _('Web sources')


class ParticipantTypeAdmin(admin.ModelAdmin):
    list_display = ('name', 'order',)
    list_editable = ('order',)


class SearchTagInline(generic.GenericTabularInline):
    model = SearchTag


class CustomMembershipAdminForm(forms.ModelForm):
    
    def clean_to_participant(self):
        to_p = self.cleaned_data['to_participant']
        from_p = self.cleaned_data['from_participant']
        if to_p == from_p:
            raise forms.ValidationError(_("A participant can't have a membership with itself. At least in this system."))
        if to_p.from_memberships.count() > 0:
            raise forms.ValidationError(_("A participant can't belong to a participant which already belongs to another participant. Things would get to complicated."))
        return to_p

class MembershipInline(admin.TabularInline):
    model = Membership
    fk_name = 'from_participant'
    form = CustomMembershipAdminForm


class IsGroupFilter(SimpleListFilter):
    title = _('Is Group')
    parameter_name = 'is_group'
    
    def lookups(self, request, model_admin):
        return (
            ('groups', _('Only groups')),
        )
    
    def queryset(self, request, queryset):
        if not self.value():
            return queryset
        return queryset.annotate(count=Count('belongs_to')).filter(count=0)


class GroupMembersFilter(SimpleListFilter):    
    title = _('Group Members')
    parameter_name = 'group_members'
    
    def lookups(self, request, model_admin):
        groups = Participant.objects.annotate(count=Count('belongs_to')).filter(count=0)
        
        lookups = []
        
        for group in groups:
            lookups.append((group.id, unicode(group),))
        return tuple(lookups)
    
    def queryset(self, request, queryset):
        if not self.value():
            return queryset
        memberships = Membership.objects.filter(to_participant=int(self.value()))
        from_participants = []
        for m in memberships:
            from_participants.append(m.from_participant.id)
        #from_participants.append(int(self.value()))
        return queryset.filter(id__in=from_participants)
    

class CustomParticipantAdminForm(forms.ModelForm):
    description = forms.CharField(widget=forms.Textarea(attrs={'class':'htmleditor'}))


class ParticipantAdmin(admin.ModelAdmin):
    actions = ['delete_selected',]
    list_display = ('name', 'type', 'order', 'is_group', 'in_num_groups', 'num_search_tags', 'num_web_sources',)
    list_editable = ('order',)
    list_filter = ('type', IsGroupFilter, GroupMembersFilter,)
    search_fields = ['name', 'description',]
    save_on_top = True
    inlines = [
        MembershipInline,
        SearchTagInline,
        WebSourceInline,
    ]
    form = CustomParticipantAdminForm
    
    def is_group(self, obj):
        if obj.from_memberships.count() == 0:
            return True
        else:
            return False
    is_group.boolean = True
    is_group.short_description = _('Is Group')
    
    def in_num_groups(self, obj):
        if obj.from_memberships.count() > 0:
            return str(obj.from_memberships.count())
        else:
            return ""
    in_num_groups.short_description = _('In number of groups')
    
    def num_search_tags(self, obj):
        return get_num_search_tags(obj)
    num_search_tags.short_description = _('Search tags')
    
    def num_web_sources(self, obj):
        return get_num_web_sources(obj)
    num_web_sources.short_description = _('Web sources')
    
    def delete_warning_msg(self, request, participant):
        msg  = _('The following associations with "%s" will be deleted') % unicode(participant)  + u': '
        msg += u'%i Events, ' % participant.related_events.count()
        msg += u'%i Documents' % participant.related_documents.count()
        messages.warning(request, msg)

    def delete_view(self, request, object_id, extra_context=None):
        self.delete_warning_msg(request, Participant.objects.get(pk=object_id))
        return super(ParticipantAdmin, self).delete_view(request, object_id, extra_context)
    
    def delete_selected(self, request, queryset):
        for object in queryset.all():
            self.delete_warning_msg(request, object)
        return delete_selected(self, request, queryset)


class CustomProjectPartAdminForm(forms.ModelForm):
    
    description = forms.CharField(widget=forms.Textarea(attrs={'class':'htmleditor'}))

    def clean_main_project_parts(self):
        data = self.cleaned_data['main_project_parts']
        for main_pp in data.all():
            if main_pp.main_project_parts.count() > 0:
                raise forms.ValidationError(_("A project part can't have a main project part which has a main project part itself. Things would get to complicated."))
        return data


class IsMainProjectPartFilter(SimpleListFilter):
    title = _('Is Main Topic')
    parameter_name = 'is_main_topic'
    
    def lookups(self, request, model_admin):
        return (
            ('main_project_parts', _('Only main topics')),
        )
    
    def queryset(self, request, queryset):
        if not self.value():
            return queryset
        return queryset.annotate(count=Count('main_project_parts')).filter(count=0)


class MainProjectPartFilter(SimpleListFilter):    
    title = _('Main Topics')
    parameter_name = 'main_topics'
    
    def lookups(self, request, model_admin):
        main_project_parts = ProjectPart.objects.annotate(count=Count('main_project_parts')).filter(count=0)
        
        lookups = []
        
        for main_pp in main_project_parts:
            lookups.append((main_pp.id, unicode(main_pp),))
        return tuple(lookups)
    
    def queryset(self, request, queryset):
        if not self.value():
            return queryset
        
        return queryset.filter(main_project_parts__in=[int(self.value()),])
        


class ProjectPartAdmin(admin.ModelAdmin):
    actions = ['delete_selected',]
    list_display = ('name', 'order', 'is_main_project_part', 'in_num_main_project_parts', 'num_search_tags', 'num_web_sources')
    list_editable = ('order',)
    list_filter = (IsMainProjectPartFilter, MainProjectPartFilter,)
    search_fields = ['name',]
    save_on_top = True
    inlines = [
        SearchTagInline,
        WebSourceInline,
    ]
    filter_horizontal = ('main_project_parts',)
    form = CustomProjectPartAdminForm
    
    def is_main_project_part(self, obj):
        if obj.main_project_parts.count() == 0:
            return True
        else:
            return False
    is_main_project_part.boolean = True
    is_main_project_part.short_description = _('Is Main Topic')
    
    def in_num_main_project_parts(self, obj):
        if obj.main_project_parts.count() > 0:
            return str(obj.main_project_parts.count())
        else:
            return ""
    in_num_main_project_parts.short_description = _('Main Topics')
    
    def num_search_tags(self, obj):
        return get_num_search_tags(obj)
    num_search_tags.short_description = _('Search tags')
    
    def num_web_sources(self, obj):
        return get_num_web_sources(obj)
    num_web_sources.short_description = _('Web sources')
    
    def delete_warning_msg(self, request, project_part):
        msg  = _('The following associations with "%s" will be deleted') % unicode(project_part)  + u': '
        msg += u'%i Events, ' % project_part.related_events.count()
        msg += u'%i Documents' % project_part.related_documents.count()
        messages.warning(request, msg)

    def delete_view(self, request, object_id, extra_context=None):
        self.delete_warning_msg(request, ProjectPart.objects.get(pk=object_id))
        return super(ProjectPartAdmin, self).delete_view(request, object_id, extra_context)
    
    def delete_selected(self, request, queryset):
        for object in queryset.all():
            self.delete_warning_msg(request, object)
        return delete_selected(self, request, queryset)


class CustomEventAdminForm(forms.ModelForm):
    description = forms.CharField(widget=forms.Textarea(attrs={'class':'htmleditor'}))


class EventAdmin(admin.ModelAdmin):
    list_display = ('title', 'event_type', 'important', 'date', 'num_project_parts', 'num_participants', 'num_search_tags', 'num_web_sources',)
    filter_horizontal = ('project_parts', 'participants',)
    inlines = [
        SearchTagInline,
        WebSourceInline,
    ]
    list_filter = ('event_type', 'important',)
    search_fields = ['title', 'description',]
    save_on_top = True
    form = CustomEventAdminForm
    
    def num_project_parts(self, obj):
        return get_num_project_parts(obj)
    num_project_parts.short_description = _('Topics')
    
    def num_participants(self, obj):
        return get_num_participants(obj)
    num_participants.short_description = _('Participants')
    
    def num_search_tags(self, obj):
        return get_num_search_tags(obj)
    num_search_tags.short_description = _('Search tags')
    
    def num_web_sources(self, obj):
        return get_num_web_sources(obj)
    num_web_sources.short_description = _('Web sources')


class QuestionAdmin(admin.ModelAdmin):
    list_display = ('title', 'answered', 'num_project_parts', 'num_participants', 'num_events', 'num_documents', 'num_web_sources',)
    search_fields = ['title',]
    inlines = [
        WebSourceInline,
    ]
    filter_horizontal = ('project_parts', 'participants', 'events', 'documents')
    save_on_top = True
    
    def num_project_parts(self, obj):
        return get_num_project_parts(obj)
    num_project_parts.short_description = _('Topics')
    
    def num_participants(self, obj):
        return get_num_participants(obj)
    num_participants.short_description = _('Participants')
    
    def num_events(self, obj):
        return get_num_events(obj)
    num_events.short_description = _('Events')
    
    def num_documents(self, obj):
        return get_num_documents(obj)
    num_documents.short_description = _('Documents')
    
    def num_web_sources(self, obj):
        return get_num_web_sources(obj)
    num_web_sources.short_description = _('Web sources')


class ProjectGoalInline(admin.StackedInline):
    model = ProjectGoal


class ProjectGoalGroupAdmin(admin.ModelAdmin):
    list_display = ('title', 'event', 'project_part', 'is_current', 'num_performance_figures',)
    save_on_top = True
    inlines = [
        ProjectGoalInline,
    ]
    
    def num_performance_figures(self, obj):
        num = obj.projectgoal_set.count()
        if num > 0:
            return str(num)
        else:
            return ""
    num_performance_figures.short_description = _('Performance figures')


class CustomDocumentAdminForm(forms.ModelForm):
    description = forms.CharField(widget=forms.Textarea(attrs={'class':'htmleditor'}))
    
    class Meta:
        exclude = ('pdf_images_generated',)


class DocumentAdmin(admin.ModelAdmin):
    list_display = ('title', 'document', 'date', 'num_project_parts', 'num_participants', 'num_events',)
    search_fields = ['title', 'description',]
    filter_horizontal = ('participants', 'project_parts', 'events',)
    save_on_top = True
    form = CustomDocumentAdminForm
    
    def num_project_parts(self, obj):
        return get_num_project_parts(obj)
    num_project_parts.short_description = _('Topics')
    
    def num_participants(self, obj):
        return get_num_participants(obj)
    num_participants.short_description = _('Participants')
    
    def num_events(self, obj):
        return get_num_events(obj)
    num_events.short_description = _('Events')


class PageAdmin(admin.ModelAdmin):
    list_display = ('document', 'number')


class SearchTagCacheEntryAdmin(admin.ModelAdmin):
    list_display = ('tag', 'document', 'num_results')


class ResearchRequestInlineFormset(forms.models.BaseInlineFormSet):
    def clean(self):
        # get forms that actually have valid data
        count = 0
        rr_content_type = ContentType.objects.get(app_label='public_project', model='question')
        for form in self.forms:
            try:
                if form.cleaned_data:
                    if form.cleaned_data['content_type'] == rr_content_type:
                        count += 1
            except AttributeError:
                # annoyingly, if a subform is invalid Django explicity raises
                # an AttributeError for cleaned_data
                pass
        if count < 1:
            raise forms.ValidationError(_('A research request must be associated with a question'))


class ResearchRequestRelationInline(admin.TabularInline):
    formset = ResearchRequestInlineFormset
    model = ResearchRequestRelation

class ResearchRequestAdmin(admin.ModelAdmin):
    list_display = ('question', 'nr', 'title', 'open', 'date_added',)
    list_filter = ('open',)
    search_fields = ['title',]
    save_on_top = True
    inlines = [
        ResearchRequestRelationInline,
    ]
    
    def question(self, obj):
        return obj.get_related_question()
    question.short_description = _('Question')


class CommentRelationInline(admin.TabularInline):
    model = CommentRelation


class CommentAdmin(admin.ModelAdmin):
    list_display = ('username', 'email', 'published', 'published_by', 'date_added',)
    list_filter = ('published','published_by',)
    search_fields = ['username', 'comment',]
    save_on_top = True
    exclude = ('activation_hash',)
    inlines = [
        CommentRelationInline,
    ]

class ActivityLogAdmin(admin.ModelAdmin):
    list_display = ('id', 'type', 'content_object', 'content_type', 'object_id', 'date')


admin.site.unregister(User)
admin.site.register(User, UserAdmin)
admin.site.register(Image, ImageAdmin)
admin.site.register(SiteConfig, SiteConfigAdmin)
admin.site.register(SiteCategory, SiteCategoryAdmin)
admin.site.register(ParticipantType, ParticipantTypeAdmin)
admin.site.register(Participant, ParticipantAdmin)
admin.site.register(ProjectPart, ProjectPartAdmin)
admin.site.register(Event, EventAdmin)
admin.site.register(Question, QuestionAdmin)
admin.site.register(ProjectGoalGroup, ProjectGoalGroupAdmin)
admin.site.register(Document, DocumentAdmin)
#admin.site.register(SearchTagCacheEntry, SearchTagCacheEntryAdmin)
#admin.site.register(Page, PageAdmin)
admin.site.register(ResearchRequest, ResearchRequestAdmin)
admin.site.register(Comment, CommentAdmin)
admin.site.register(ActivityLog, ActivityLogAdmin)



import datetime
from django.core.files import File
from django.contrib.sites.models import Site
from public_project.models import *



class ExampleData:

    def create_images(self):
        self.i1 = Image(
            title = 'Bruegel - Tower of Babel',
            attribution = 'Pieter Bruegel the Elder - The Tower of Babel (Vienna) - PD Licence, Wikimedia Commons',
            attribution_url = 'http://commons.wikimedia.org/wiki/File:Pieter_Bruegel_the_Elder_-_The_Tower_of_Babel_(Vienna)_-_Google_Art_Project.jpg', 
        )
        img = File(open('ep_setup_app/imgs/Bruegel_Tower_of_Babel.jpg', 'rw'))
        self.i1.image = img
        self.i1.save()

    def create_site_config(self):
        self.site = Site.objects.all()[0]
        self.site.domain = '127.0.0.1:8075'
        self.site.name = '127.0.0.1:8075'
        self.site.save()
        
        self.sc = SiteConfig(
            title          = 'Tower of Babel',
            short_title    = 'Tower of Babel',
            intro_text     = '<p><strong>We will finish. Someday.</strong></p>\r\n<p>' + \
                             '<img src="../../../../media/images/Bruegel_Tower_of_Babel_1.jpg" ' + \
                             'alt="Bruegel - Tower of Babel" /></p>\r\n<p>This project is taking a litte ' + \
                             'longer than expected. Having some language problems as well. ' + \
                             'But we will finish. We promise.</p>',
            about_text     = "This project website is run by Tower of Babel International, London, " + \
                             "but is not at all associated with the Babel Tower Coorporation, Antalya or the " + \
                             "Babel Tower Group, Jamaica.",
            footer         = "For questions ask Jim, John, Miranda or Turtle-Joey.",
            contact_text   = "Contact: no-reply@towerofbabelinternationalassociation.com",
        )
        self.sc.save()
    
    
    def create_site_categories(self):
        self.scat1 = SiteCategory(
            category = 'home',
            intro_text = '"whose top may reach unto heaven" - that is the tower we want to build. ' + \
                         'Some may call us crazy. Some may call us naive. But we are determined and ' + \
                         'beside from having some language problems between some of our workers ' + \
                         'we are very much in our plan and will succeed!',    
        )
        self.scat1.save()
        
        self.scat2 = SiteCategory(
            category = 'project_parts',
            intro_text = 'We want to make everything as transparent as possible. Our trust ' + \
                         'in us is our trust in you.',    
        )
        self.scat2.save()
        
        self.scat3 = SiteCategory(
            category = 'questions',
            intro_text = 'We try to work with you on common questions around the tower building here.',    
        )
        self.scat3.save()
        
        self.scat4 = SiteCategory(
            category = 'participants',
            intro_text = 'Such an ambitious project has various actors involved. You can find them here.',    
        )
        self.scat4.save()
        
        self.scat5 = SiteCategory(
            category = 'goals',
            intro_text = 'We have adopted our goals over the centuries. See them here.',    
        )
        self.scat5.save()
        
        self.scat6 = SiteCategory(
            category = 'events',
            intro_text = 'This is the building timeline of our amazing construction project.',    
        )
        self.scat6.save()
        
        self.scat7 = SiteCategory(
            category = 'documents',
            intro_text = 'We want that you are able to take part. So we publish all construction documents here.',    
        )
        self.scat7.save()
    
    
    def create_project_parts(self):
        self.pp1 = ProjectPart(
            name = 'Stone Management',
            description = 'Management of stone acquistion, quality management and deposition.',
        )
        self.pp1.save()
        
        self.pp1_spp1 = ProjectPart(
            name = 'Stone Acquisition',
            description = 'Management of stone acquistion.',
        )
        self.pp1_spp1.save()
        self.pp1_spp1.main_project_parts.add(self.pp1)
        self.pp1_spp1.save()
        
        self.pp1_spp2 = ProjectPart(
            name = 'Quality management',
            description = 'Observation of stone format, material density.',
        )
        self.pp1_spp2.save()
        self.pp1_spp2.main_project_parts.add(self.pp1)
        self.pp1_spp2.save()
        
        self.pp1_spp3 = ProjectPart(
            name = 'Stone Deposition',
            description = 'Deposition of stones.',
        )
        self.pp1_spp3.save()
        self.pp1_spp3.main_project_parts.add(self.pp1)
        self.pp1_spp3.save()
        
        self.pp2 = ProjectPart(
            name = 'Certification',
            description = 'Certificates for stone measures, stone hole size compliance and stone authenticity.'
        )
        self.pp2.save()
        
        self.pp3 = ProjectPart(
            name = 'Construction Work',
            description = 'Construction work taking place, floor management.'
        )
        self.pp3.save()
        #pp2.main_project_parts = [pp1,],
        #pp2.save()
    
    
    def create_participants(self):
        
        self.pt1 = ParticipantType(
            name = 'Building Companies',
            order = 100,
        )
        self.pt1.save()
        
        self.pt2 = ParticipantType(
            name = 'Administration',
            order = 200,
        )
        self.pt2.save()
        
        self.pt3 = ParticipantType(
            name = 'Working Groups',
            order = 300,
        )
        self.pt3.save()
        
        self.p1 = Participant(
            name             = 'Tower of Babel International',
            description      = 'We are the makers. We are the doers. The sky is the limit.',
            type = self.pt1,
        )
        self.p1.save()

        self.p2 = Participant(
            name             = 'Babel Tower Coorporation',
            description      = 'Former project responsibles. Hmm. Never met. Strange people.',
            type = self.pt1,
        )
        self.p2.save()
        
        self.p3 = Participant(
            name = 'Department of Stone Measurement (DOFSTON)',
            description = 'Responsible for all stone measurement certificates at building site',
            type = self.pt2,
        )
        self.p3.save()
        
        self.st2 = SearchTag(
            name = 'DOFSTON',
            content_object = self.p3,
        )
        self.st2.save()
        
        self.st3 = SearchTag(
            name = 'Department of Stone Measurement',
            content_object = self.p3,
        )
        self.st3.save()
        
        self.p4 = Participant(
            name = 'Mrs. Smith',
            description = 'Very geeky lady from the DOFSTON, but kind of nice.',
        )
        self.p4.save()
        
        self.p5 = Participant(
            name = 'Working Group "The Working Wombats"',
            description = 'Working group.',
            type = self.pt3,
        )
        self.p5.save()
        
        self.p6 = Participant(
            name = 'Working Group "The Building Bees"',
            description = 'Another working group.',
            type = self.pt3,
        )
        self.p6.save()
        
        self.ws1 = WebSource(
            title = 'Official Website',
            url = 'http://www.spiegel.de',
            content_object = self.p4,
        )
        self.ws1.save()
        
        self.st1 = SearchTag(
            name = 'Smith',
            content_object = self.p4,
        )
        self.st1.save()
        
        self.m1 = Membership(
            from_participant = self.p4,
            to_participant = self.p3,
            function = 'Department Manager',
        )
        self.m1.save()
        
        self.c1 = Comment(
            username = 'Robert Gutherfeather',
            email = 'robgus@smirdenbroeth.co.uk',
            comment = 'Is Mrs. Smith THE Mrs. Smith I encountered in Durbanport in 1957?? How exciting!',
            published = True,
        )
        self.c1.save()
        
        self.cr1 = CommentRelation(
            comment = self.c1,
            content_object = self.p4,
        )
        self.cr1.save()
        
    
    
    def create_events(self):
        self.e1 = Event(
            title       = 'Begin of the construction work',
            event_type  = 'MI',
            important   = True,
            description = 'Start of the construction of the tower, great atmosphere, everybody was there ' + \
                          'and there was this spirit in the air, you could feel it, you know? ' + \
                          'Lots of tears, our Meta-CEO said a few words ("They are one people and have one ' + \
                          'language, and nothing will be withheld from them which they purpose to do.", whew), ' + \
                          'very emotional.',
            date        = '2002-05-12',
        )
        self.e1.save()
        self.e1.participants = [self.p2,]
        self.e1.save()
        
        self.e3 = Event(
            title = 'Mrs. Smith made a telephone call',
            event_type = 'IN',
            description = "On this day Mrs. Smith made a telephone call. Normally this woman never answers " + \
                          "the phone so this is obviously quite dubious.",
            date = '2005-03-22',
        )
        self.e3.save()
        self.e3.participants = [self.p4,]
        self.e3.project_parts = [self.pp1,]
        self.e3.save()
        
        self.e2 = Event(
            title = 'No DOFSTON certificate, rebuilding of 500 first floors',
            event_type = 'IN',
            important   = True,
            description = "The Department of Stone Measurements doesn't give us the certificate for the first " + \
                          "500 floors due to irregularities in stone dimensions, we are forced to rebuild.",
            date = '2005-07-15',
        )
        self.e2.save()
        self.e2.participants = [self.p2, self.p3,]
        self.e2.project_parts = [self.pp2,]
        self.e2.save()
        
        
        self.e4 = Event(
            title = 'Milestone of first 2.500 floors reached, big party',
            event_type = 'IN',
            important   = True,
            description = "The first 2.500 floors are finally build. Everything looks very solid and stable. " + \
                          "There was a big party on floor 1.745.",
            date = '2007-04-18',
        )
        self.e4.save()
        self.e4.participants = [self.p2, self.p3,]
        self.e4.project_parts = [self.pp3,]
        self.e4.save()
        
        self.e5 = Event(
            title = 'Language problems, rebuilding floors 1.000-2.5000',
            event_type = 'IN',
            important   = True,
            description = 'Some misunderstandings  between "The Working Wombats" and "The Building Bees" ' + \
                          'led to floor 2.500 (luxury apartments) beeing build on ' + \
                          'floor 1.000. Rebuilding of floors 1.000-2.500 necessary.',
            date = '2007-06-24',
        )
        self.e5.save()
        self.e5.participants = [self.p5, self.p6,]
        self.e5.project_parts = [self.pp3,]
        self.e5.save()
    
    
    def create_project_goals(self):
        self.pgg1 = ProjectGoalGroup(
            title       = 'Initial Goals',
            event       = self.e1,
            is_current = False,
            description = 'We want to build a really high tower, the sky is the limit, at lease 5000 floors tall',
        )
        self.pgg1.save()

        self.pgg1_pg1 = ProjectGoal(
            project_goal_group = self.pgg1,
            name  = 'Floors',
            performance_figure = '5000+',
            order = 100,
        )
        self.pgg1_pg1.save()
        
        self.pgg1_pg2 = ProjectGoal(
            project_goal_group = self.pgg1,
            name = 'Beautiness',
            performance_figure = 'very beautiful',
            order = 200,
        )
        self.pgg1_pg2.save()
        
        self.pgg2 = ProjectGoalGroup(
            title       = 'Adopted Goals',
            event       = self.e2,
            is_current = False,
            description = 'Adoption of goal after stone dimension certificate issues.',
        )
        self.pgg2.save()

        self.pgg2_pg1 = ProjectGoal(
            project_goal_group = self.pgg2,
            name  = 'Floors',
            performance_figure = '4000+',
            order = 100,
        )
        self.pgg2_pg1.save()
        
        self.pgg2_pg2 = ProjectGoal(
            project_goal_group = self.pgg2,
            name = 'Beautiness',
            performance_figure = 'rather beautiful',
            order = 200,
        )
        self.pgg2_pg2.save()
        
        self.pgg3 = ProjectGoalGroup(
            title       = 'Adopted Goals (2nd time)',
            event       = self.e5,
            is_current = True,
            description = 'Adoption of goal after language issues.',
        )
        self.pgg3.save()

        self.pgg3_pg1 = ProjectGoal(
            project_goal_group = self.pgg3,
            name  = 'Floors',
            performance_figure = '3000+',
            order = 100,
        )
        self.pgg3_pg1.save()
        
        self.pgg3_pg2 = ProjectGoal(
            project_goal_group = self.pgg3,
            name = 'Beautiness',
            performance_figure = 'not too shabby',
            order = 200,
        )
        self.pgg3_pg2.save()
    
    def create_documents(self):
        self.d1 = Document(
            title =       'DOFSTON Certificate Refusal',
            date =        '2005-06-23',
            description = 'Writing from Department of Stone Measurements about stone block certificate refusal.', 
        )
        pdf = File(open('ep_setup_app/docs/dofston_cert_refusal.pdf', 'rw'))
        self.d1.document = pdf
        self.d1.save()
        self.d1.project_parts = [self.pp2,]
        self.d1.participants = [self.p3,]
        self.d1.save()
        
        self.d2 = Document(
            title =       'Transcript of telephone call Mrs. Smith',
            date =        '2005-03-25',
            description = 'Mrs. Smith made a phone call. Transcript was sent by anonymous caller.', 
        )
        pdf = File(open('ep_setup_app/docs/phonecall_mrs_smith.pdf', 'rw'))
        self.d2.document = pdf
        self.d2.save()
        self.d2.project_parts = [self.pp1,]
        self.d2.events = [self.e3,]
        self.d2.participants = [self.p4,]
        self.d2.save()
    
    
    def create_questions(self):
        self.q1 = Question(
            title = 'Available stone deposits in the area',
            description = 'Due to the necessary rebuilding of the first 500 floors we are in need of stone ' + \
                          'deposition sites for temporarily shelving stones with wrong dimensions.',
        )
        self.q1.save()
        self.q1.project_parts = [self.pp1,]
        self.q1.participants = [self.p2, self.p3,]
        self.q1.save()
        
        self.q2 = Question(
            title = 'Investigation of misunderstandings',
            description = 'Misunderstandings between two working groups lead to the necessity to rebuild ' + \
                          'more than 1.000 floors. We want to figure out more about the circumstances.',
        )
        self.q2.save()
        self.q2.project_parts = [self.pp3,]
        self.q2.participants = [self.p5, self.p6,]
        self.q2.events = [self.e5,]
        self.q2.save()
    
    
    def create_research_requests(self):
        self.rr1 = ResearchRequest(
            nr = 'R1',
            title = 'PDF downloads of worker group protocols',
            open = True,
            description = 'Until the end of 2006 the protocols of worker group meetings were publicly ' + \
                          'available on worker group websites but were taken down afterwards. If someone ' + \
                          'still has PDF downloads of the protocols please get in touch with us.',
        )
        self.rr1.save()
        
        self.rrr1 = ResearchRequestRelation(
            research_request = self.rr1,
            content_object = self.q2,
        )
        self.rrr1.save()
    
    def create(self):
        self.create_images()
        self.create_site_config()
        self.create_site_categories()
        self.create_project_parts()
        self.create_participants()
        self.create_events()
        self.create_project_goals()
        self.create_documents()
        self.create_questions()
        self.create_research_requests()



import unittest

def build_test_suite_from(test_cases):
    """
    Returns a single or group of unittest test suite(s) that's ready to be
    run. The function expects a list of classes that are subclasses of
    TestCase.

    The function will search the module where each class resides and
    build a test suite from that class and all subclasses of it.
    """
    test_suites = []
    for test_case in test_cases:
        mod = __import__(test_case.__module__)
        components = test_case.__module__.split('.')
        for comp in components[1:]:
            mod = getattr(mod, comp)
        tests = []
        for item in mod.__dict__.values():
            if type(item) is type and issubclass(item, test_case):
                tests.append(item)
        test_suites.append(unittest.TestSuite(map(unittest.TestLoader().loadTestsFromTestCase, tests)))
    return unittest.TestSuite(test_suites)


import re
from django.db.models import Q
from public_project.models import Page


#PublicDocs
def normalize_query(query_string,
                    findterms=re.compile(r'"([^"]+)"|(\S+)').findall,
                    normspace=re.compile(r'\s{2,}').sub):
    ''' Splits the query string in invidual keywords, getting rid of unecessary spaces
        and grouping quoted words together.
        Example:
        
        >>> normalize_query('  some random  words "with   quotes  " and   spaces')
        ['some', 'random', 'words', 'with quotes', 'and', 'spaces']
    
    '''
    return [normspace(' ', (t[0] or t[1]).strip()) for t in findterms(query_string)] 


#PublicDocs
def get_query(query_string, search_fields):
    ''' Returns a query, that is a combination of Q objects. That combination
        aims to search keywords within a model by testing the given search fields.
    
    '''
    query = None # Query to search for every search term        
    #terms = normalize_query(query_string)
    for term in [query_string]: # Using complete query string instead of separated terms
        or_query = None # Query to search for a given term in each field
        for field_name in search_fields:
            q = Q(**{"%s__icontains" % field_name: term})
            if or_query is None:
                or_query = q
            else:
                or_query = or_query | q
        if query is None:
            query = or_query
        else:
            query = query & or_query
    return query


def search_for_documents(query_string):
    entry_query = get_query(query_string, ['document__title', 'content',])
    found_pages = Page.objects.select_related().filter(entry_query).order_by('document','number')
        
    document_list = []
    for page in found_pages:
        if page.document not in document_list:
            page.document.search_tags = [query_string,]
            document_list.append(page.document)
    
    return document_list
     